<?php

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

// Return if PMS is not active
if( ! defined( 'PMS_VERSION' ) ) return;


Class PMS_Multiple_Currencies_Admin {

    public function __construct(){

        /**
         * Subscription Plans interface
         */

        // Add Subscription Plan MC Settings
        add_action( 'pms_view_meta_box_subscription_details_price_bottom', array( $this, 'plan_options_output' ) );

        // Add the modal for entering prices in different currencies
        add_action( 'pms_view_meta_box_subscription_details_bottom', array( $this, 'plan_prices_modal_content' ) );

        // Save Subscription Plan MC Prices
        add_action( 'wp_ajax_pms_mc_plan_prices_save', array( $this, 'plan_prices_save_action') );

        // Save Subscription Plan "Don't Switch Currency" Option
        add_action( 'pms_save_meta_box_pms-subscription', array( $this, 'save_no_currency_switch_option') );

        /**
         * Settings
         */

        // Add settings page content
        add_action( 'pms-settings-page_payments_after_subtabs', array( $this, 'settings_page_content' ) );

        // Add settings page navigation link
        add_action( 'pms-settings-page_payments_sub_tab_navigation_items', array( $this, 'settings_page_navigation' ) );

        /**
         * Payments Interface
         */

        // Modify Payments List table Amount column so it reflects the correct currency
        add_filter( 'pms_payments_list_table_column_amount', array( $this, 'payments_modify_payments_list_table_amount_column' ), 20, 2 );

        // Modify Currency Symbol on the Edit payment page
        add_filter( 'pms_add_new_edit_payment_currency_symbol', array( $this, 'payments_modify_edit_screen_currency_symbol' ), 20, 2 );

        // Modify last payment Currency in Subscription Billing Schedule section
        add_filter( 'pms_last_payment_currency', array( $this, 'last_payment_currency' ), 10, 2 );

        // Modify Member Payments List table
        add_filter( 'pms_member_payments_list_table_entry_data', array( $this, 'payments_modify_member_payments_list_table_data' ), 20, 2 );

        // Add payment logs
        add_filter( 'pms_payment_logs_system_error_messages', array( $this, 'payments_add_custom_log_system_messages' ), 20, 2 );

        /**
         * Discount Codes
         */

        // Add Discount Code "Currency Availability" Option
        add_filter( 'pms_view_meta_box_discount_codes_bottom', array( $this, 'discounts_add_currency_availability_selection' ) );

        // Save Discount Code "Currency Availability" option
        add_action( 'pms_save_meta_box_pms-discount-codes', array( $this, 'discounts_save_currency_availability' ), 20 );

        /**
         * Reports
         */

        // Output JS Data for Reports page Chart
        add_action( 'pms_output_content_submenu_page_pms-reports-page', array( $this, 'reports_output_chart_js_data' ) );

        // Add Reports page Summary section MC Tab navigation link
        add_filter( 'pms_reports_summary_tabs', array( $this, 'reports_summary_mc_tab_link' ) );

        // Add Reports page Summary section MC Tab content
        add_action( 'pms_reports_summary_sections_bottom', array( $this, 'reports_summary_mc_tab_content' ), 10, 3 );

        // Remove default JS Data output. JS Data for Multiple Currencies is now outputted.
        global $pms_submenu_page_reports;
        remove_action( 'admin_print_footer_scripts', array( $pms_submenu_page_reports, 'output_chart_js_data' ) );

    }

    /**
     *  Add Subscription Plan MC Settings
     *
     * @param $plan_id
     * @return void
     *
     */
    public function plan_options_output( $plan_id ){

        // don't add the Multiple Currencies options if the Subscription Plan is free and has no sign-up fee
        $plan = pms_get_subscription_plan( $plan_id );

        if ( $plan->price == 0 && !$plan->has_sign_up_fee() )
            return;

        $payment_settings = get_option( 'pms_payments_settings', array() );

        if ( $payment_settings['conversion_type'] === 'manual' ) {
            echo '<div class="cozmoslabs-form-field-wrapper">';

                echo '<label class="cozmoslabs-form-field-label">';
                    echo esc_html__( 'Multiple Currencies', 'paid-member-subscriptions' );
                    echo '<a href="https://www.cozmoslabs.com/docs/paid-member-subscriptions/add-ons/multiple-currencies/?utm_source=wpbackend&utm_medium=pms-documentation&utm_campaign=PMSDocs" target="_blank" data-code="f223" class="pms-docs-link dashicons dashicons-editor-help"></a>';
                echo '</label>';

                echo '<a class="pms-mc-plan-prices-modal-trigger button button-secondary" href="#">' . esc_html__( 'Edit Prices for Other Currencies', 'paid-member-subscriptions' ) . '</a>';
                echo '<p class="cozmoslabs-description cozmoslabs-description-space-left">' . esc_html__( 'Specify a custom price for each available currency.', 'paid-member-subscriptions' ) . '</p>';

            echo '</div>';
        }

        $value = get_post_meta( $plan_id, 'pms_plan_no_currency_switch', true );

        ob_start();
        ?>
            <div class="pms-meta-box-field-wrapper cozmoslabs-form-field-wrapper cozmoslabs-toggle-switch">

                <label for="pms-plan-no-currency-switch" class="pms-meta-box-field-label cozmoslabs-form-field-label"><?php esc_html_e( 'Don\'t Switch Currency','paid-member-subscriptions' ); ?></label>

                <div class="cozmoslabs-toggle-container">
                    <input type="checkbox" id="pms-plan-no-currency-switch" name="pms_plan_no_currency_switch" <?php checked( 1, $value ) ?>  value="1">
                    <label class="cozmoslabs-toggle-track" for="pms-plan-no-currency-switch"></label>
                </div>

                <div class="cozmoslabs-toggle-description">
                    <label for="pms-plan-no-currency-switch" class="cozmoslabs-description"><?php esc_html_e( 'Restrict the sale of this plan to the default currency only.', 'paid-member-subscriptions' ); ?></label>
                </div>

            </div>

        <?php

        $output = ob_get_contents();
        ob_end_clean();

        echo $output; //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
    }

    /**
     * Save Subscription Plan "Don't Switch Currency" Option
     *
     * @return void
     *
     */
    public function save_no_currency_switch_option(){

        if( empty( $_POST['post_ID'] ) )
            return;

        if( isset( $_POST['pms_plan_no_currency_switch'] ) )
            update_post_meta( absint( $_POST['post_ID'] ), 'pms_plan_no_currency_switch', sanitize_text_field( $_POST['pms_plan_no_currency_switch'] ) );
        else if( !isset( $_POST['pms_plan_no_currency_switch'] ) )
            update_post_meta( absint( $_POST['post_ID'] ), 'pms_plan_no_currency_switch', '' );

    }

    /**
     * Add the modal for entering prices in different currencies
     *
     * @param $subscription_plan_id
     * @return void
     *
     */
    public function plan_prices_modal_content( $subscription_plan_id ){
        $payment_settings = get_option( 'pms_payments_settings', array() );

        if ( $payment_settings['conversion_type'] !== 'manual' )
            return;

        $plan      = pms_get_subscription_plan( $subscription_plan_id );
        $mc_prices = pms_mc_get_prices( $subscription_plan_id );

        ?>
            <div id="pms-modal-multiple-currencies" class="pms-mc-modal" title="Multiple Currencies">
                <div class="pms-modal__holder">

                    <p class="cozmoslabs-description"><?php printf( esc_html__( 'Enter the prices for this plan in different currencies. The available currencies are determined based on the settings from the %sSettings → Payments → Multiple Currencies%s page.', 'paid-member-subscriptions' ), '<a href="'. esc_url( admin_url( 'admin.php?page=pms-settings-page&tab=payments&nav_sub_tab=payments_multiple_currencies' ) ) .'">', '</a>' ); ?></p>
                    <p class="cozmoslabs-description"><?php esc_html_e( 'If a currency is defined but a plan does not have a specific price set for that currency, the value and currency of the original price will be used.', 'paid-member-subscriptions' ); ?></p>
                    <p class="cozmoslabs-description"><?php printf( esc_html__( 'Enable the %sAuto Convert Prices%s option from the %sSettings → Payments → Multiple Currencies%s page to convert prices automatically.', 'paid-member-subscriptions' ), '<strong>', '</strong>', '<a href="'. esc_url( admin_url( 'admin.php?page=pms-settings-page&tab=payments&nav_sub_tab=payments_multiple_currencies#payments-multiple-currencies' ) ) .'">', '</a>' ); ?></p>

                    <h3 style="margin-top: 35px;"><?php esc_html_e( 'Prices in different currencies', 'paid-member-subscriptions' ); ?></h3>

                    <div class="pms-modal__fullrow">
                        <?php
                            if( !$plan->has_sign_up_fee() )
                                printf( '<div class="pms-modal__currency">%s: <strong>%s</strong></div>', esc_html( $payment_settings['currency'] ), esc_html( $plan->price ) );
                            else
                                printf( '<div class="pms-modal__currency">%s: <strong>%s</strong></div> <span class="pms-modal__price-separator">+</span> <div class="pms-modal__sign-up-fee">Sign-up fee: <strong>%s</strong></div>', esc_html( $payment_settings['currency'] ), esc_html( $plan->price ), esc_html( $plan->sign_up_fee ) );
                        ?>
                    </div>

                    <?php if( !empty( $payment_settings['multiple-currencies'] ) ) : ?>
                        <div class="pms-mc-modal__entries">
                            <?php foreach( $payment_settings['multiple-currencies'] as $currency ) : ?>

                                <div class="pms-modal__fullrow">
                                    <div class="pms-modal__currency">
                                        <span class="pms-modal__label"><?php echo esc_html( $currency ) . ': '; ?></span>
                                        <input type="number" class="pms-modal__price" name="pms_multiple_currency_price_<?php echo esc_html( $currency ); ?>" data-currency="<?php echo esc_html( $currency ); ?>" value="<?php echo ( !empty( $mc_prices[ $currency ]['price'] ) ? esc_html( $mc_prices[ $currency ]['price'] ) : '' );?>" />
                                    </div>

                                    <?php if( $plan->has_sign_up_fee() ) : ?>
                                        <span class="pms-modal__price-separator"><?php esc_html_e( ' + ', 'paid-member-subscriptions' ); ?></span>
                                        <div class="pms-modal__sign-up">
                                            <span><?php printf( esc_html__( 'Sign-up fee: ', 'paid-member-subscriptions' ) ); ?></span>
                                            <input type="number" class="pms-modal__sign-up-fee" name="pms_multiple_currency_sign_up_fee_<?php echo esc_html( $currency ); ?>" data-currency="<?php echo esc_html( $currency ); ?>" value="<?php echo ( !empty( $mc_prices[ $currency ]['sign_up_fee'] ) ? esc_html( $mc_prices[ $currency ]['sign_up_fee'] ) : '' );?>" />
                                        </div>
                                    <?php endif; ?>

                                    <?php if( function_exists( 'pms_in_pwyw_pricing_enabled' ) && pms_in_pwyw_pricing_enabled( $subscription_plan_id ) ) : ?>
                                        <div class="pms-modal__min-sub-price">
                                            <span><?php printf( esc_html__( 'Min Price: ', 'paid-member-subscriptions' ) ); ?></span>
                                            <input type="number" class="pms-modal__min-price" name="pms_multiple_currency_min-price_<?php echo esc_html( $currency ); ?>" data-currency="<?php echo esc_html( $currency ); ?>" value="<?php echo ( !empty( $mc_prices[ $currency ]['min_price'] ) ? esc_html( $mc_prices[ $currency ]['min_price'] ) : '' );?>" />
                                        </div>

                                        <div class="pms-modal__max-sub-price">
                                            <span><?php printf( esc_html__( 'Max Price: ', 'paid-member-subscriptions' ) ); ?></span>
                                            <input type="number" class="pms-modal__max-price" name="pms_multiple_currency_max-price_<?php echo esc_html( $currency ); ?>" data-currency="<?php echo esc_html( $currency ); ?>" value="<?php echo ( !empty( $mc_prices[ $currency ]['max_price'] ) ? esc_html( $mc_prices[ $currency ]['max_price'] ) : '' );?>" />
                                        </div>
                                    <?php endif; ?>

                                </div>

                            <?php endforeach; ?>
                        </div>
                    <?php endif; ?>

                    <div class="pms-modal__bottom">
                        <input type="hidden" name="pmstkn" value="<?php echo esc_attr( wp_create_nonce( 'pms-mc-save-prices' ) ) ?>" />
                        <input type="hidden" name="pms_subscription_plan_id" value="<?php echo esc_attr( $subscription_plan_id ) ?>" />

                        <a class="button button-primary pms-modal__save">
                            <?php esc_html_e( 'Save Prices', 'paid-member-subscriptions' ) ?>
                        </a>

                        <div class="pms-mc-modal__sucess">
                            <?php esc_html_e( 'Prices saved!', 'paid-member-subscriptions' ); ?>
                        </div>
                    </div>
                </div>
            </div>
        <?php
    }

    /**
     * Save Subscription Plan MC Prices
     *
     * @return void
     *
     */
    public function plan_prices_save_action(){

        check_ajax_referer( 'pms-mc-save-prices', 'pmstkn' );

        if( empty( $_POST['pms_mc_values'] ) || empty( $_POST['pms_subscription_plan_id'] ) )
            die();

        $mc_values = pms_array_sanitize_text_field( $_POST['pms_mc_values'] ); //phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
        $values = array();

        foreach( $mc_values as $currency => $currency_price_data ){

            if( !empty( $currency_price_data['price'] ) )
                $values[$currency]['price'] = (float)$currency_price_data['price'];
            else $values[$currency]['price'] = '';

            if( !empty( $currency_price_data['sign_up_fee'] ) )
                $values[$currency]['sign_up_fee'] = (float)$currency_price_data['sign_up_fee'];
            else $values[$currency]['sign_up_fee'] = '';

            if( !empty( $currency_price_data['min_price'] ) )
                $values[$currency]['min_price'] = (float)$currency_price_data['min_price'];
            else $values[$currency]['min_price'] = '';

            if( !empty( $currency_price_data['max_price'] ) )
                $values[$currency]['max_price'] = (float)$currency_price_data['max_price'];
            else $values[$currency]['max_price'] = '';

        }

        if( !empty( $values ) ){

            if( update_post_meta( absint( $_POST['pms_subscription_plan_id'] ), 'pms_mc_prices', $values ) )
                die('success');

        }

        die();

    }

    /**
     * Add Multiple Currencies options to PMS Settings -> Payments
     *
     * @param $payment_settings
     * @return void
     *
     */
    public function settings_page_content( $payment_settings ){

        $active_sub_tab = ( ! empty( $_GET['nav_sub_tab'] ) ? sanitize_text_field( $_GET['nav_sub_tab'] ) : 'payments_general' );

        ?>
        <div id="payments-multiple-currencies" class="cozmoslabs-sub-tab cozmoslabs-sub-tab-multiple-currencies <?php echo ( $active_sub_tab == 'payments_multiple_currencies' ? 'tab-active' : '' ); ?>" data-sub-tab-slug="payments_multiple_currencies">

            <div class="cozmoslabs-form-subsection-wrapper" id="cozmoslabs-subsection-multiple-currency-configs">
                <h4 class="cozmoslabs-subsection-title">
                    <?php esc_html_e( 'Multiple Currencies', 'paid-member-subscriptions' ); ?>
                    <a href=" https://www.cozmoslabs.com/docs/paid-member-subscriptions/add-ons/multiple-currencies/?utm_source=wpbackend&utm_medium=pms-documentation&utm_campaign=PMSDocs" target="_blank" data-code="f223" class="pms-docs-link dashicons dashicons-editor-help"></a>
                </h4>

                <div class="cozmoslabs-form-field-wrapper">
                    <label class="cozmoslabs-form-field-label"><?php esc_html_e( 'Conversion Type', 'paid-member-subscriptions' ); ?></label>

                    <div class="cozmoslabs-radio-inputs-row" id="payments-conversion-type">

                        <label for="payments-conversion-type-manual">
                            <input type="radio" id="payments-conversion-type-manual" value="manual" <?php if( empty( $payment_settings['conversion_type'] ) || $payment_settings['conversion_type'] == 'manual' ) echo 'checked="checked"'; ?> name="pms_payments_settings[conversion_type]">
                            <?php esc_html_e( 'Manual', 'paid-member-subscriptions' ); ?>
                        </label>

                        <label for="payments-conversion-type-automatic">
                            <input type="radio" id="payments-conversion-type-automatic" value="automatic" <?php if( !empty( $payment_settings['conversion_type'] ) && $payment_settings['conversion_type'] == 'automatic' ) echo 'checked="checked"'; ?> name="pms_payments_settings[conversion_type]">
                            <?php esc_html_e( 'Automatic', 'paid-member-subscriptions' ); ?>
                        </label>

                    </div>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left" id="manual-type-description"><?php esc_html_e( 'Prices will be displayed according to the custom price set for each currency in the Subscription Plan settings.', 'paid-member-subscriptions' ); ?></p>
                    <p class="cozmoslabs-description cozmoslabs-description-space-left" id="automatic-type-description"><?php esc_html_e( 'Prices will be automatically converted from the default currency to the selected one.', 'paid-member-subscriptions' ); ?></p>
                </div>

                <div class="cozmoslabs-form-field-wrapper">
                    <label class="cozmoslabs-form-field-label"><?php esc_html_e( 'Default Currency', 'paid-member-subscriptions' ) ?></label>
                    <strong><?php echo esc_html( $payment_settings['currency'] ); ?></strong>
                </div>

                <div class="cozmoslabs-form-field-wrapper">
                    <label class="cozmoslabs-form-field-label" for="extra-currency"><?php esc_html_e( 'Extra Currencies', 'paid-member-subscriptions' ) ?></label>

                    <select id="extra-currency" class="pms-chosen" name="pms_payments_settings[multiple-currencies][]" multiple>
                        <?php
                        $currencies = pms_get_currencies();

                        if( isset( $currencies[ $payment_settings['currency'] ] ) )
                            unset( $currencies[ $payment_settings['currency'] ] );

                        foreach( $currencies as $currency_code => $currency )
                            echo '<option value="' . esc_attr( $currency_code ) . '" ' . ( isset( $payment_settings['multiple-currencies'] ) && is_array( $payment_settings['multiple-currencies'] ) && in_array( $currency_code, $payment_settings['multiple-currencies'] ) ? 'selected' : '' ) . '>' . esc_html( $currency ) . ' (' . esc_html( $currency_code ) . ')</option>';
                        ?>
                    </select>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Choose any additional currencies you would like to use, apart from the default one.', 'paid-member-subscriptions' ) ?></p>
                </div>

                <div class="cozmoslabs-form-field-wrapper">
                    <label class="cozmoslabs-form-field-label" for="restricted-currency"><?php esc_html_e( 'PayPal Restricted Currencies', 'paid-member-subscriptions' ) ?></label>

                    <?php if( !empty( $payment_settings['multiple-currencies'] ) ) : ?>
                        <select id="restricted-currency" class="pms-chosen" name="pms_payments_settings[restricted-paypal-currencies][]" multiple>
                            <?php
                            foreach( $payment_settings['multiple-currencies'] as $currency )
                                echo '<option value="' . esc_attr( $currency ) . '" ' . ( ( isset( $payment_settings['restricted-paypal-currencies'] ) && is_array( $payment_settings['restricted-paypal-currencies'] ) && in_array( $currency, $payment_settings['restricted-paypal-currencies'] ) ) ? 'selected' : '' ) . '>' . esc_html( $currencies[ $currency ] ) . ' (' . esc_html( $currency ) . ')</option>';
                            ?>
                        </select>
                    <?php else : ?>
                        <p class="cozmoslabs-description"><?php esc_html_e( 'First define some extra currencies above and save the settings in order to setup these currencies.', 'paid-member-subscriptions' ) ?></p>
                    <?php endif; ?>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left">                  <?php
                        printf(
                            esc_html__( 'Certain currencies cannot be accepted through PayPal due to Merchant Restrictions. %sView PayPal supported currencies%s', 'paid-member-subscriptions' ),
                            '<a style="font-weight: 500;" href="https://developer.paypal.com/docs/reports/reference/paypal-supported-currencies/" target="_blank">',
                            '</a>'
                        );
                        ?>
                    </p>
                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Select which Extra Currency cannot be received in PayPal. Payments will convert to the default currency before processing, as long as an active currency Converter is available to support the conversion.', 'paid-member-subscriptions' ) ?></p>
                </div>

                <div class="cozmoslabs-form-field-wrapper cozmoslabs-toggle-switch" id="payments-currency-switcher">
                    <label class="cozmoslabs-form-field-label" for="currency-switcher"><?php esc_html_e( 'Currency Switcher', 'paid-member-subscriptions' ) ?></label>

                    <div class="cozmoslabs-toggle-container">
                        <input type="checkbox" id="currency-switcher" name="pms_payments_settings[currency_switcher]" value="yes" <?php echo ( isset( $payment_settings['currency_switcher'] ) && $payment_settings['currency_switcher'] === 'yes' ) ? 'checked' : '' ?> />
                        <label class="cozmoslabs-toggle-track" for="currency-switcher"></label>
                    </div>
                    <div class="cozmoslabs-toggle-description">
                        <label for="currency-switcher" class="cozmoslabs-description"><?php esc_html_e( 'Enable the Manual Currency Switcher.', 'paid-member-subscriptions' ); ?></label>
                    </div>
                </div>

                <div class="cozmoslabs-form-field-wrapper cozmoslabs-toggle-switch" id="payments-user-location">
                    <label class="cozmoslabs-form-field-label" for="currency-switcher"><?php esc_html_e( 'User Location', 'paid-member-subscriptions' ) ?></label>

                    <div class="cozmoslabs-toggle-container">
                        <input type="checkbox" id="user-location" name="pms_payments_settings[user_location]" value="yes" <?php echo ( isset( $payment_settings['user_location'] ) && $payment_settings['user_location'] === 'yes' ) ? 'checked' : '' ?> />
                        <label class="cozmoslabs-toggle-track" for="user-location"></label>
                    </div>
                    <div class="cozmoslabs-toggle-description">
                        <label for="user-location" class="cozmoslabs-description"><?php esc_html_e( 'Detect user location.', 'paid-member-subscriptions' ); ?></label>
                    </div>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Show prices according to the user\'s location.', 'paid-member-subscriptions' ) ?></p>
                </div>

                <div class="cozmoslabs-form-field-wrapper cozmoslabs-toggle-switch" id="payments-auto-convert-prices">
                    <label class="cozmoslabs-form-field-label" for="auto-convert-prices"><?php esc_html_e( 'Auto Convert Prices', 'paid-member-subscriptions' ) ?></label>

                    <div class="cozmoslabs-toggle-container">
                        <input type="checkbox" id="auto-convert-prices" name="pms_payments_settings[auto_convert_prices]" value="yes" <?php echo ( isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] === 'yes' ) ? 'checked' : '' ?> />
                        <label class="cozmoslabs-toggle-track" for="auto-convert-prices"></label>
                    </div>
                    <div class="cozmoslabs-toggle-description">
                        <label for="auto-convert-prices" class="cozmoslabs-description"><?php esc_html_e( 'Automatically convert Subscription Plan prices', 'paid-member-subscriptions' ); ?></label>
                    </div>
                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Automatically convert prices when a Subscription Plan price is not set for the required currency.', 'paid-member-subscriptions' ); ?></p>
                </div>
            </div>

            <div class="cozmoslabs-form-subsection-wrapper" id="cozmoslabs-subsection-currency-converter-configs">
                <h4 class="cozmoslabs-subsection-title">
                    <?php esc_html_e( 'Currency Conversion', 'paid-member-subscriptions' ); ?>
                </h4>

                <div class="cozmoslabs-form-field-wrapper" id="pms-converter-list">
                    <label class="cozmoslabs-form-field-label" for="convertor-api-provider"><?php esc_html_e( 'Converter', 'paid-member-subscriptions' ) ?></label>

                    <select id="convertor-api-provider" class="" name="pms_payments_settings[convertor_api_provider]">

                        <option value=""><?php esc_html_e( 'Select Converter...', 'paid-member-subscriptions' ) ?></option>
                        <option value="PMS_Exchange_Currency_Converter" <?php echo ( isset( $payment_settings['convertor_api_provider'] ) && $payment_settings['convertor_api_provider'] === 'PMS_Exchange_Currency_Converter' ) ? 'selected' : ''; ?>><?php esc_html_e( 'Exchange Converter', 'paid-member-subscriptions' ); ?></option>
                        <option value="PMS_Frankfurter_Currency_Converter" <?php echo ( isset( $payment_settings['convertor_api_provider'] ) && $payment_settings['convertor_api_provider'] === 'PMS_Frankfurter_Currency_Converter' ) ? 'selected' : ''; ?>><?php esc_html_e( 'Frankfurter Converter', 'paid-member-subscriptions' ); ?></option>
                        <option value="PMS_Alpha_Vantage_Currency_Converter" <?php echo ( isset( $payment_settings['convertor_api_provider'] ) && $payment_settings['convertor_api_provider'] === 'PMS_Alpha_Vantage_Currency_Converter' ) ? 'selected' : ''; ?>><?php esc_html_e( 'Alpha Vantage', 'paid-member-subscriptions' ); ?></option>

                    </select>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Choose your preferred Converter.', 'paid-member-subscriptions' ) ?></p>
                    <p class="cozmoslabs-description cozmoslabs-description-space-left pms-converter-description" id="PMS_Exchange_Currency_Converter" style="display: none"><?php echo wp_kses_post( __( 'To get your API key, visit <a href="https://app.exchangerate-api.com" target="_blank">Exchange Converter</a> and follow the steps provided.', 'paid-member-subscriptions' ) ); ?></p>
                    <p class="cozmoslabs-description cozmoslabs-description-space-left pms-converter-description" id="PMS_Alpha_Vantage_Currency_Converter" style="display: none"><?php echo wp_kses_post( __( 'To get your API key, visit <a href="https://www.alphavantage.co/support/#api-key" target="_blank">Alpha Vantage</a> and follow the easy steps provided.', 'paid-member-subscriptions' ) ); ?></p>

                </div>

                <div class="cozmoslabs-form-field-wrapper pms-converter-api-key" id="pms-exchange-api-key">
                    <label class="cozmoslabs-form-field-label" for="exchange-api-key"><?php esc_html_e( 'API Key', 'paid-member-subscriptions' ); ?></label>
                    <input type="text" id="exchange-api-key" value="<?php echo isset( $payment_settings['exchange_api_key'] ) ? esc_html( $payment_settings['exchange_api_key'] ) : '' ?>" name="pms_payments_settings[exchange_api_key]" placeholder="Enter Your API Key">
                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Enter the API key provided by the Exchange Converter.', 'paid-member-subscriptions' ) ?></p>
                </div>

                <div class="cozmoslabs-form-field-wrapper pms-converter-api-key" id="pms-alpha-vantage-api-key">
                    <label class="cozmoslabs-form-field-label" for="alpha-vantage-api-key"><?php esc_html_e( 'API Key', 'paid-member-subscriptions' ); ?></label>
                    <input type="text" id="alpha-vantage-api-key" value="<?php echo isset( $payment_settings['alpha_vantage_api_key'] ) ? esc_html( $payment_settings['alpha_vantage_api_key'] ) : '' ?>" name="pms_payments_settings[alpha_vantage_api_key]">
                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Enter the API key provided by the Alpha Vantage Converter.', 'paid-member-subscriptions' ) ?></p>
                </div>

                <div class="cozmoslabs-form-field-wrapper">
                    <label class="cozmoslabs-form-field-label" for="conversion-update-interval"><?php esc_html_e( 'Update Interval', 'paid-member-subscriptions' ) ?></label>

                    <select id="conversion-update-interval" class="" name="pms_payments_settings[conversion_update_interval]">

                        <option value="hourly" <?php echo ( isset( $payment_settings['conversion_update_interval'] ) && $payment_settings['conversion_update_interval'] == 'hourly' ) ? 'selected' : ''; ?>><?php esc_html_e( '1 hour', 'paid-member-subscriptions' ); ?></option>
                        <option value="every_6_hours" <?php echo ( isset( $payment_settings['conversion_update_interval'] ) && $payment_settings['conversion_update_interval'] == 'every_6_hours' ) ? 'selected' : ''; ?>><?php esc_html_e( '6 hours', 'paid-member-subscriptions' ); ?></option>
                        <option value="twicedaily" <?php echo ( !isset( $payment_settings['conversion_update_interval'] ) || $payment_settings['conversion_update_interval'] == 'twicedaily' ) ? 'selected' : ''; ?>><?php esc_html_e( '12 hours', 'paid-member-subscriptions' ); ?></option>
                        <option value="daily" <?php echo ( isset( $payment_settings['conversion_update_interval'] ) && $payment_settings['conversion_update_interval'] == 'daily' ) ? 'selected' : ''; ?>><?php esc_html_e( '24 hours', 'paid-member-subscriptions' ); ?></option>

                    </select>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Specify how often the conversion rates should be updated.', 'paid-member-subscriptions' ) ?></p>
                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Before setting the interval, make sure to check how often the selected Converter updates the rates to avoid making unnecessary requests.', 'paid-member-subscriptions' ) ?></p>
                </div>

                <?php if (
                        !empty( $payment_settings['convertor_api_provider'] ) &&
                        ( $payment_settings['convertor_api_provider'] === 'PMS_Frankfurter_Currency_Converter' ||
                            ( $payment_settings['convertor_api_provider'] === 'PMS_Exchange_Currency_Converter' && !empty( $payment_settings['exchange_api_key'] ) ) ||
                            ( $payment_settings['convertor_api_provider'] === 'PMS_Alpha_Vantage_Currency_Converter' && !empty( $payment_settings['alpha_vantage_api_key'] ) ) )
                    ) : ?>

                    <div class="cozmoslabs-form-field-wrapper">
                        <label class="cozmoslabs-form-field-label"><?php esc_html_e( 'Conversion Rates', 'paid-member-subscriptions' ); ?></label>

                        <button id="pms-update-conversion-rates" class="button button-secondary"><?php esc_html_e( 'Update Rates', 'paid-member-subscriptions' ); ?></button>

                        <?php $request_info = get_option( 'pms_currency_exchange_request_date' ); ?>
                        <p><?php esc_html_e( 'Updated: ', 'paid-member-subscriptions' );?> <strong><span id="pms-request-date"><?php echo isset( $request_info ) ? esc_html( $request_info ) : ''; ?></span></strong></p>

                    </div>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Update the rates to improve performance and reduce future requests.', 'paid-member-subscriptions' );?></p>

                    <p class="cozmoslabs-description cozmoslabs-description-space-left" style="margin-top: 10px;">
                        <span style="display: none; margin: 10px; color: green;" class="pms-request-success-message"><?php esc_html_e( 'Conversion rates updated successfully!', 'paid-member-subscriptions' ); ?></span>
                        <span style="display: none; margin-left: 10px; color: red;" class="pms-request-error-message"><?php esc_html_e( 'Something went wrong!', 'paid-member-subscriptions' ); ?></span>
                    </p>

                <?php endif; ?>

            </div>

                <?php if( !empty( $payment_settings['multiple-currencies'] ) ) : ?>
                    <div class="cozmoslabs-form-subsection-wrapper" id="cozmoslabs-subsection-currency-labels">
                        <div id="cozmoslabs-currency-labels">
                            <h4 class="cozmoslabs-subsection-title"><?php esc_html_e( 'Currency Labels', 'paid-member-subscriptions' ) ?></h4>
                            <p class="cozmoslabs-description"><?php esc_html_e( 'Define custom labels or symbols for any additional currencies in use.', 'paid-member-subscriptions' ); ?></p>

                            <?php foreach( $payment_settings['multiple-currencies'] as $currency ) : ?>

                                <div class="cozmoslabs-form-field-wrapper">
                                    <label class="cozmoslabs-form-field-label" for="<?php echo esc_html( $currency ) ?>-label"><?php echo sprintf( '%s Label', esc_html( $currency ) ) ?></label>
                                    <input type="text" id="<?php echo esc_html( $currency ) ?>-label" name="pms_payments_settings[multiple-currencies-labels][<?php echo esc_html( $currency ) ?>]" value="<?php echo isset( $payment_settings['multiple-currencies-labels'][$currency] ) ? esc_html( $payment_settings['multiple-currencies-labels'][$currency] ) : '' ?>">
                                </div>

                            <?php endforeach; ?>
                        </div>
                    </div>
                <?php endif; ?>

        </div>
        <?php
    }

    /**
     * Add Multiple Currencies navigation link to PMS Settings -> Payments
     *
     * @param $payment_settings
     * @return void
     *
     */
    public function settings_page_navigation( $payment_settings ){

        $active_sub_tab = ( ! empty( $_GET['nav_sub_tab'] ) ? sanitize_text_field( $_GET['nav_sub_tab'] ) : 'payments_general' );

    ?>
        <li class="subsubsub-sub-tab">
            <a id="nav-sub-tab-multiple-currencies"
               data-sub-tab-slug="payments_multiple_currencies"
               href="<?php echo esc_url( admin_url( add_query_arg( array( 'page' => 'pms-settings-page', 'tab' => 'payments', 'nav_sub_tab' => 'payments_multiple_currencies' ), 'admin.php' ) ) ); ?>"
               class="nav-sub-tab <?php echo ( $active_sub_tab == 'payments_multiple_currencies' ? 'current' : '' ) ?>"><?php esc_html_e( 'Multiple Currencies', 'paid-member-subscriptions' ); ?>
            </a>
        </li>
    <?php
    }

    /**
     * Modify Payments List table Amount column so it reflects the correct currency
     *
     * @param $output
     * @param $item
     * @return string
     *
     */
    public function payments_modify_payments_list_table_amount_column( $output, $item ){

        $payment_currency = pms_mc_get_payment_currency( $item['id'] );

        if ( !empty( $item['discount_code'] ) ) {
            $output = '<span class="pms-has-bubble">';

            $output .= pms_format_price( $item['amount'], !empty( $payment_currency ) ? $payment_currency : pms_get_active_currency() ) . '<span class="pms-discount-dot"> % </span>';

            $output .= '<div class="pms-bubble">';
                $output .= '<div><span class="alignleft">' . esc_html__('Discount code', 'paid-member-subscriptions') . '</span><span class="alignright">' . esc_html( $item['discount_code'] ) . '</span></div>';
            $output .= '</div>';

            $output .= '</span>';
        } else
            $output = pms_format_price( $item['amount'], !empty( $payment_currency ) ? $payment_currency : pms_get_active_currency() );

        return $output;

    }

    /**
     * Modify Currency Symbol on the Edit payment page
     *
     * @param $currency_symbol
     * @param $payment_id
     * @return mixed|string
     *
     */
    public function payments_modify_edit_screen_currency_symbol( $currency_symbol, $payment_id ){

        if( empty( $payment_id ) )
            return $currency_symbol;

        $payment_currency = pms_mc_get_payment_currency( $payment_id );

        if( !empty( $payment_currency ) )
            return pms_get_currency_symbol( $payment_currency );

        return $currency_symbol;

    }

    /**
     * Modify last payment Currency in Subscription Billing Schedule section
     *
     * @param $currency
     * @param $form_data
     * @return mixed|string
     *
     */
    public function last_payment_currency( $currency, $form_data ){
        $payments = pms_get_payments( array( 'user_id' => $form_data['user_id'], 'subscription_plan_id' => $form_data['subscription_plan_id'], 'number' => 1 ) );

        if( isset( $payments[0] ) && !empty( $payments[0]->id ) )
            $payment_id = $payments[0]->id;
        else
            $payment_id = 0;

        return pms_mc_get_payment_currency( $payment_id );
    }

    /**
     * Modify Member Payments List table
     *
     * @param $data
     * @param $payment
     * @return mixed
     *
     */
    public function payments_modify_member_payments_list_table_data( $data, $payment ){
        $payment_currency = pms_mc_get_payment_currency( $payment->id );

        if( !empty( $payment_currency ) )
            $data['amount'] = pms_get_currency_symbol( $payment_currency ) . $payment->amount;

        return $data;
    }

    /**
     * Add payment logs
     *
     * @param $message
     * @param $log
     * @return mixed|string
     *
     */
    public function payments_add_custom_log_system_messages( $message, $log ){

        if ( empty( $log['type'] ) )
            return $message;

        $kses_args = array(
            'strong' => array()
        );

        switch( $log['type'] ) {
            case 'pms_mc_currency_switched':
                $message = sprintf( __( 'Checkout currency switched to <strong>%s</strong> based on the automatically detected user location and the user selected currency.', 'paid-member-subscriptions' ), $log['data']['currency'] );
                break;
            case 'pms_mc_currency_converted':
                $message = sprintf( __( 'Extra currency price of <strong>%s</strong> converted to <strong>%s</strong>.', 'paid-member-subscriptions' ), $log['data']['from'], $log['data']['to'] );
                break;
        }

        return wp_kses( $message, $kses_args );

    }

    /**
     * Add Discount Code "Currency Availability" Option
     *
     * @param $discount_id
     * @return void
     *
     */
    public function discounts_add_currency_availability_selection( $discount_id ){

        if( empty( $discount_id ) )
            return;

        $payment_settings = get_option( 'pms_payments_settings', array() );
        $discount         = new PMS_IN_Discount_Code( $discount_id );
        $discount_meta    = PMS_IN_Discount_Codes_Meta_Box::get_discount_meta_by_code( $discount->code );

        $currency_availability = isset( $discount_meta['pms_discount_currency_availability'] ) ? maybe_unserialize( $discount_meta['pms_discount_currency_availability'][0] ) : array();

        if( !empty( $payment_settings['multiple-currencies'] ) ) : ?>
            <div class="pms-meta-box-field-wrapper cozmoslabs-form-field-wrapper cozmoslabs-checkbox-list-wrapper">

                <?php wp_nonce_field('pms_discount_codes_currency_nonce', 'pms_mc_nonce'); ?>

                <label for="pms-discount-currency-availability" class="pms-meta-box-field-label cozmoslabs-form-field-label">
                    <?php esc_html_e( 'Currency availability','paid-member-subscriptions' ); ?>
                    <a href="https://www.cozmoslabs.com/docs/paid-member-subscriptions/add-ons/multiple-currencies/?utm_source=wpbackend&utm_medium=pms-documentation&utm_campaign=PMSDocs" target="_blank" data-code="f223" class="pms-docs-link dashicons dashicons-editor-help"></a>
                </label>

                <div class="cozmoslabs-checkbox-list">

                    <div class="cozmoslabs-chckbox-container">
                        <input type="checkbox" id="pms-discount-currency-availability-<?php echo esc_html( $payment_settings['currency'] ); ?>" name="pms_discount_currency_availability[]" value="<?php echo esc_html( $payment_settings['currency'] ) ?>" <?php echo ( !empty( $currency_availability ) && in_array( $payment_settings['currency'], $currency_availability ) ? 'checked' : '' ) ?>>
                        <label class="pms-meta-box-checkbox-label" for="pms-discount-currency-availability-<?php echo esc_html( $payment_settings['currency'] ); ?>"><?php echo esc_html( $payment_settings['currency'] ); ?></label>
                    </div>

                    <?php foreach( $payment_settings['multiple-currencies'] as $currency ) : ?>
                        <div class="cozmoslabs-chckbox-container">
                            <input type="checkbox" id="pms-discount-currency-availability-<?php echo esc_html( $currency ); ?>" name="pms_discount_currency_availability[]" value="<?php echo esc_html( $currency ) ?>" <?php echo ( !empty( $currency_availability ) && in_array( $currency, $currency_availability ) ? 'checked' : '' ) ?>>
                            <label class="pms-meta-box-checkbox-label" for="pms-discount-currency-availability-<?php echo esc_html( $currency ); ?>"><?php echo esc_html( $currency ); ?></label>
                        </div>
                    <?php endforeach; ?>

                </div>

                <p class="cozmoslabs-description cozmoslabs-description-space-left"><?php esc_html_e( 'Choose the currencies for which the discount code will be valid. If none selected, the code will apply to all.', 'paid-member-subscriptions' ); ?></p>

            </div>
        <?php
        endif;
    }

    /**
     * Save Discount Code "Currency Availability" option
     *
     * @param $post_id
     * @return void
     *
     */
    public function discounts_save_currency_availability( $post_id ){

        // Verify nonce before anything
        if( !isset( $_POST['pms_mc_nonce'] ) || !wp_verify_nonce( sanitize_text_field( $_POST['pms_mc_nonce'] ), 'pms_discount_codes_currency_nonce' ) )
            return;

        if( isset( $_POST['pms_discount_currency_availability'] ) ) {
            $discount_currency_availability = pms_array_sanitize_text_field( $_POST['pms_discount_currency_availability'] ); //phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
            update_post_meta( $post_id, 'pms_discount_currency_availability', $discount_currency_availability );
        }
        else update_post_meta( $post_id, 'pms_discount_currency_availability', array() );

    }

    /**
     * Output JS Data for Reports page Chart
     *
     * @return void
     *
     */
    public function reports_output_chart_js_data(){

        $reports_data = $this->reports_get_chart_data();

        // Start ouput
        echo '<script type="text/javascript">';

        echo 'var pms_chart_labels = ' . wp_json_encode( $reports_data['pms_chart_labels'] ) . ';';
        echo 'var pms_chart_data = ' . wp_json_encode( $reports_data['js_data'] ) . ';';
        echo 'var pms_default_currency = ' . wp_json_encode( $reports_data['default_currency'] ) . ';';
        echo 'var pms_default_currency_symbol = "' . esc_html( html_entity_decode( pms_get_currency_symbol( $reports_data['default_currency'] ) ) ) . '";';

        echo '</script>';

    }

    /**
     * Prepare JS Data for Reports page Chart
     *
     * @return array
     * @throws DateMalformedStringException
     *
     */
    public function reports_get_chart_data() {
        global $pms_submenu_page_reports;

        $payment_settings = get_option( 'pms_payments_settings', array() );
        $default_currency = pms_get_active_currency();
        $payments         = $pms_submenu_page_reports->queried_payments;
        $data             = array();
        $pms_chart_labels = array();

        if( isset( $_GET['pms-filter-time'] ) && $_GET['pms-filter-time'] == 'custom_date' ){

            // Difference in days
            $difference = ( strtotime( $pms_submenu_page_reports->end_date ) - strtotime( $pms_submenu_page_reports->start_date ) ) / ( 60 * 60 * 24 );

            if ( $difference <= 1 ) {
                $date_format = 'G';
            } elseif ( $difference <= 31 ) {
                $date_format = 'j';
            } else {
                $date_format = 'n';
            }

        }
        elseif ( isset( $_GET['pms-filter-time'] ) && ( $_GET['pms-filter-time'] == 'this_year' || $_GET['pms-filter-time'] == 'last_year' ) ) {
            $date_format = 'n';
        }
        elseif ( isset( $_GET['pms-filter-time'] ) && ( $_GET['pms-filter-time'] == 'today' || $_GET['pms-filter-time'] == 'yesterday' ) ) {
            $date_format = 'G';
        }
        else {
            $date_format = 'j';
        }

        $first_day = new DateTime( $pms_submenu_page_reports->start_date );
        $first_day = $first_day->format( $date_format );

        $last_day  = new DateTime( $pms_submenu_page_reports->end_date );
        $last_day  = $last_day->format( $date_format );

        for( $i = $first_day; $i <= $last_day; $i++ ) {
            if( !isset( $data[$i] ) )
                $data[$i] = array();

            if( !in_array( $i, $pms_chart_labels ) )
                $pms_chart_labels[] = (string)$i;
        }

        // amount paid in other currencies
        $other_currencies_in_usd = 0;

        foreach( $payments as $payment ){

            $currency = pms_mc_get_payment_currency( $payment->id );

            if( empty( $currency ) )
                $currency = $payment_settings['currency'];

            $payment_date = new DateTime( $payment->date );

            if( !isset( $data[ $payment_date->format( $date_format ) ][$currency] ) )
                $data[ $payment_date->format( $date_format ) ][$currency] = array( 'earnings' => 0, 'payments' => 0 );

            $data[ $payment_date->format( $date_format ) ][$currency]['earnings'] += $payment->amount;
            $data[ $payment_date->format( $date_format ) ][$currency]['payments'] += 1;

            $currency             = pms_mc_get_payment_currency( $payment->id );
            $base_currency_amount = pms_get_payment_meta( $payment->id, 'base_currency_amount', true );

            if( !empty( $base_currency_amount ) && !empty( $currency ) ){
                $other_currencies_in_usd = $other_currencies_in_usd + $base_currency_amount;
            }

        }

        // prepare data for JS
        $js_data = array();

        foreach( $data as $day_key => $day ){

            foreach( $day as $currency => $item ){

                if( !isset( $js_data[$currency] ) )
                    $js_data[$currency] = array();

                $js_data[ $currency ]['earnings'][ $day_key ] = $item['earnings'];
                $js_data[ $currency ]['payments'][ $day_key ] = $item['payments'];

            }

        }

        foreach( $js_data as $currency => $item ){

            for( $i = $first_day; $i <= $last_day; $i++ ) {
                if( !isset( $js_data[$currency]['earnings'][$i] ) )
                    $js_data[$currency]['earnings'][$i] = 0;

                if( !isset( $js_data[$currency]['payments'][$i] ) )
                    $js_data[$currency]['payments'][$i] = 0;
            }

            ksort( $js_data[$currency]['earnings'] );
            ksort( $js_data[$currency]['payments'] );

        }

        if ( empty( $js_data ) ) {
            for( $i = $first_day; $i <= $last_day; $i++ ) {
                $js_data[ $default_currency ]['earnings'][$i] = 0;
                $js_data[ $default_currency ]['payments'][$i] = 0;
            }
        }


        $reports_data = array(
            'pms_chart_labels' => $pms_chart_labels,
            'default_currency' => $default_currency,
            'js_data' => $js_data,
        );

        return $reports_data;
    }

    /**
     * Add Reports page Summary section MC Tab navigation link
     *
     * @param $tabs
     * @return array
     *
     */
    public function reports_summary_mc_tab_link( $tabs ) {
        $other_currencies_tab = array(
                'other-currencies' => array(
                    'id' => 'pms-other-currencies-link',
                    'title' => esc_html__( 'Other Currencies', 'paid-member-subscriptions' ),
                    'section_class' => 'pms-other-currencies-section'
                ),
        );

        return array_merge( $tabs, $other_currencies_tab );
    }

    /**
     * Add Reports page Summary section MC Tab content
     *
     * @param $summary_data
     * @param $results_arrow
     * @param $previous
     * @return void
     *
     */
    public function reports_summary_mc_tab_content( $summary_data, $results_arrow, $previous ) {
        include 'views/view-reports-summary-tab.php';
    }

}

new PMS_Multiple_Currencies_Admin;
