jQuery(function ($) {

    // Hide/Show Conversion Type descriptions and specific fields as needed
    handleConversionTypeField();
    $('#payments-conversion-type').click(function (e) {
        handleConversionTypeField();
    });

    // Hide/Show Converter description and API Key Field as needed
    handleConverterField();
    $('#convertor-api-provider').change(function() {
        $('.pms-converter-description').hide();
        handleConverterField();
    });

    // Hide/Show User Location Field as needed
    handleUserLocationField();
    $('#currency-switcher').click(function (e) {
        handleUserLocationField();
    });


    // Handle Subscription Plan settings Multiple Currencies modal
    $(document).on('click', '.pms-mc-plan-prices-modal-trigger', function (event) {
        event.preventDefault();
        displayModal();
    })

    $(document).on('click', '.pms-modal__close', function () {
        $('.pms-modal').hide()
    })

    $(document).on('click', '.pms-mc-modal .pms-modal__save', function () {

        var values = {}

        $('.pms-mc-modal__entries .pms-modal__fullrow .pms-modal__price').each( function( index, item ){

            values[$(item).data('currency')] = { price: $(item).val(), sign_up_fee : '', min_price: '', max_price: '' }

        })

        $('.pms-mc-modal__entries .pms-modal__fullrow .pms-modal__sign-up-fee').each( function( index, item ){

            values[$(item).data('currency')]['sign_up_fee'] = $(item).val()

        })

        $('.pms-mc-modal__entries .pms-modal__fullrow .pms-modal__min-price').each( function( index, item ){

            values[$(item).data('currency')]['min_price'] = $(item).val()

        })

        $('.pms-mc-modal__entries .pms-modal__fullrow .pms-modal__max-price').each( function( index, item ){

            values[$(item).data('currency')]['max_price'] = $(item).val()

        })

        var data = {
            action                  : 'pms_mc_plan_prices_save',
            pmstkn                  : $('.pms-mc-modal .pms-modal__bottom input[name="pmstkn"]').val(),
            pms_subscription_plan_id: $('.pms-mc-modal .pms-modal__bottom input[name="pms_subscription_plan_id"]').val(),
            pms_mc_values           : values,
        }

        $.post(ajaxurl, data, function (response) {

            if( response == 'success' ){
                $('.pms-mc-modal .pms-modal__bottom .pms-mc-modal__sucess').show().fadeOut( 2000 )
            }

        })
    })


    $('#pms-update-conversion-rates').on('click', function(event) {
        event.preventDefault();

        let button = $(this);
        button.prop('disabled', true);

        $.ajax({
            type: 'POST',
            url: ajaxurl,
            data: {
                action: 'pms_mc_data_update_request'
            },
            success: function(response) {
                if ( response && response.hasOwnProperty('text') )
                    $('#pms-request-date').html(response['text']);

                if ( response && response.hasOwnProperty('type') ) {
                    if (response['type'] === 'success')
                        $('.pms-request-success-message').fadeIn(400).delay(2000).fadeOut(400);
                    else $('.pms-request-error-message').fadeIn(400).delay(2000);
                }
            },
            complete: function() {
                button.prop('disabled', false);
            }
        });


    });

    // Handle Extra Currencies and PayPal Restricted Currencies fields Chosen initialization
    handleChosenFields();
    $('#nav-sub-tab-multiple-currencies').on('click', function(event) {
        setTimeout(function() {
            handleChosenFields();
        }, 100);
    });

})

/**
 * Handle Subscription Plan settings Multiple Currencies modal
 *
 * @returns {boolean}
 */
function displayModal() {
    jQuery('.pms-mc-modal').dialog({
        resizable: false,
        height: 'auto',
        width: 800,
        modal: true,
        closeOnEscape: true,
        open: function () {
            jQuery('.ui-widget-overlay').bind('click',function () {
                jQuery('.pms-mc-modal').dialog('close');
            })
        },
        close: function () {

        }
    });
    return false;
}

/**
 * Hide/Show Conversion Type descriptions and specific fields as needed
 *
 */
function handleConversionTypeField() {
    if ( jQuery('#payments-conversion-type-manual').is(':checked') ) {
        jQuery('#payments-auto-convert-prices').show();
        jQuery('#manual-type-description').show();
        jQuery('#automatic-type-description').hide();
    }
    else {
        jQuery('#payments-auto-convert-prices').hide();
        jQuery('#manual-type-description').hide();
        jQuery('#automatic-type-description').show();
    }
}

/**
 * Hide/Show Converter description and API Key Field as needed
 *
 */
function handleConverterField() {
    let converter = jQuery('#convertor-api-provider');

    if ( converter.val() !== '' && converter.val() !== 'PMS_Frankfurter_Currency_Converter' ) {
        jQuery('#' + jQuery('#convertor-api-provider').val()).show();

        if ( converter.val() === 'PMS_Exchange_Currency_Converter' ) {
            jQuery('.pms-converter-api-key').hide();
            jQuery('#pms-exchange-api-key').show();
        }
        else if ( converter.val() === 'PMS_Alpha_Vantage_Currency_Converter' ) {
            jQuery('.pms-converter-api-key').hide();
            jQuery('#pms-alpha-vantage-api-key').show();
        }
    }
    else jQuery('.pms-converter-api-key').hide();
}

/**
 * Hide/Show User Location Field as needed
 *
 */
function handleUserLocationField() {
    if ( jQuery('#currency-switcher').is(':checked') )
        jQuery('#payments-user-location').show();
    else jQuery('#payments-user-location').hide();
}

/**
 * Handle Extra Currencies and PayPal Restricted Currencies fields Chosen initialization
 */
function handleChosenFields() {

    if (jQuery.fn.chosen !== undefined) {
        let currencySelectors = jQuery('#extra-currency, #restricted-currency');

        currencySelectors.each( function() {
            let selector = jQuery(this);

            selector.chosen('destroy');

            selector.chosen({
                search_contains: true,
                placeholder_text_multiple: "Select Currencies"
            });
        });

    }

}