<?php

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

// Return if PMS is not active
if( ! defined( 'PMS_VERSION' ) ) return;

Class PMS_Multiple_Currencies {

    public function __construct(){

        // Set the currency data cookie
		add_action( 'template_redirect', array( $this, 'setup_currency_data_cookie' ), 1 );

        // Front-end price output
        add_filter( 'pms_subscription_plan_output_price', array( $this, 'modify_price_output' ), 20, 2 );

        // Front-end sign-up fee output
        add_filter( 'pms_subscription_plan_output_sign_up_fee', array( $this, 'modify_signup_fee_output' ), 20, 2 );

        // Front-end currency switcher output
        add_filter( 'pms_fd_subscription_plan_list_title', array( $this, 'output_register_form_currency_switcher' ) );
        add_action( 'pms_register_form_subscription_plans_field_before', array( $this, 'output_register_form_currency_switcher' ) );
        add_action( 'pms_new_subscription_form_subscription_plans_field_before', array( $this, 'output_register_form_currency_switcher' ) );
        add_action( 'pms_change_subscription_message_extra_info', array( $this, 'output_change_form_currency_switcher' ), 10, 4 );
        add_filter( 'pms_renew_subscription_before_form', array( $this, 'output_renew_retry_form_currency_switcher' ), 10, 3 );
        add_filter( 'pms_retry_payment_subscription_confirmation_message', array( $this, 'output_renew_retry_form_currency_switcher' ), 10, 3 );

        add_filter( 'pms_pb_subscription_plans_field_title', array( $this, 'output_pb_form_currency_switcher' ), 10, 2 );

        // Modify payment data before it's inserted into the database
        add_filter( 'pms_process_checkout_payment_data', array( $this, 'modify_checkout_payment_data' ), 10, 2 );

        // Modify PSP recurring data
        add_filter( 'pms_process_checkout_subscription_data', array( $this, 'modify_checkout_subscription_data' ), 20, 2 );

        // Save custom currency to payment
        add_filter( 'pms_register_payment_data', array( $this, 'save_custom_currency_to_payment' ), 20, 2 );

        // Modify currency for the PWYW Subscription
        add_filter( 'pms_pwyw_currency_before_output', array( $this, 'modify_pwyw_subscription_currency' ), 10, 2 );
        add_filter( 'pms_pwyw_subscription_price_range_currency', array( $this, 'modify_pwyw_subscription_currency' ), 10, 2 );

        // Modify PWYW price based on MC Currency set price
        add_filter( 'pms_pwyw_price_before_output', array( $this, 'modify_pwyw_subscription_price' ), 10, 2 );

        // Modify PWYW min_price and max_price for price range validation
        add_filter( 'pms_pwyw_subscription_min_price', array( $this, 'modify_pwyw_subscription_min_price' ), 10, 2 );
        add_filter( 'pms_pwyw_subscription_max_price', array( $this, 'modify_pwyw_subscription_max_price' ), 10, 2 );

        // Modify currency in the Subscription Price merge tag
        add_filter( 'pms_merge_tag_subscription_price_currency', array( $this, 'modify_subscription_price_tag_currency' ), 20, 2 );

        // Modify amount in the Subscription Price merge tag
        add_filter( 'pms_merge_tag_subscription_price_amount', array( $this, 'modify_subscription_price_tag_amount' ), 20, 3 );

        // Modify currency in the Payment History shortcode
        add_filter( 'pms_payment_history_amount_row_currency', array( $this, 'modify_payment_history_amount_row_currency' ), 20, 2 );
        add_filter( 'pms_tax_breakdown_in_payment_history_shortcode_tooltip_currency', array( $this, 'modify_payment_history_amount_row_currency' ), 20, 2 );

        // Modify Subscription Plan amount in Stripe Intents checkout
        add_filter( 'pms_stripe_create_payment_intent_amount', array( $this, 'modify_stripe_checkout_amount' ), 20, 2 );

        // Modify Subscription Plan amount in Stripe Connect checkout
        add_filter( 'pms_stripe_calculate_payment_amount', array( $this, 'modify_stripe_checkout_amount' ), 20, 2 );

        // Disable Stripe sign-up fee calculation when prices are being changed as we add it ourselves
        add_filter( 'pms_stripe_create_payment_intent_apply_sign_up_fee', array( $this, 'disable_stripe_sign_up_fee' ), 20, 2 );

        // Modify currency in Stripe Intents args
        add_filter( 'pms_stripe_create_payment_intent_args', array( $this, 'modify_stripe_intent_args_currency' ), 20, 2 );
        add_filter( 'pms_stripe_process_payment_args', array( $this, 'modify_stripe_intent_process_payment_args_currency' ) );

        // Modify currency in Stripe Connect args
        add_filter( 'pms_stripe_connect_update_payment_intent_args', array( $this, 'modify_stripe_intent_args_currency' ), 20, 2 );

        //Modify currency in Stripe Connect -> update payment intent
        add_filter( 'pms_stripe_connect_update_payment_intent_currency', array( $this, 'modify_stripe_connect_update_payment_intent_currency' ), 20, 3 );

        // Modify currency in Next Payment Date account display
        add_filter( 'pms_account_next_payment_date_currency', array( $this, 'modify_next_payment_date_currency' ), 20, 2 );

        // Modify currency in Member Subscription payment Data
        add_filter( 'pms_cron_process_member_subscriptions_payment_data', array( $this, 'modify_psp_payments_currency' ), 20, 2 );

        // Save currency to payment meta for PSP added payments
        add_action( 'pms_cron_after_processing_member_subscription', array( $this, 'save_currency_to_payment_meta_for_psp_payments' ), 1, 2 );

        // Modify Currency on PayPal Express Confirmation form
        add_filter( 'pms_paypal_express_confirmation_form_currency', array( $this, 'set_currency_on_paypal_express_confirmation_form' ), 20, 2 );

        // Modify currency in Discount Codes message
        add_filter( 'pms_dc_success_message_currency', array( $this, 'modify_discounts_success_message_currency' ), 20, 2 );

        // Modify Subscription Plan price in Discount Code message
        add_filter( 'pms_dc_success_message_plan_price', array( $this, 'modify_discounts_success_message_plan_price' ), 20, 2 );
        add_filter( 'pms_dc_success_message_recurring_plan_price', array( $this, 'modify_discounts_success_message_plan_price' ), 20, 2 );

        // Validate discount code for the currency that is being used
        add_action( 'pms_dc_get_discount_error', array( $this, 'validate_discounts_currency_availability' ), 20, 4 );

        // Save currency on subscription when it's inserted
        add_action( 'pms_after_inserting_subscription_data_inside_db', array( $this, 'save_currency_to_subscription' ), 20, 4 );

        // Filter currency symbols so it shows the symbol for some currencies and the ISO slug for others
        add_filter( 'pms_currency_symbols', array( $this, 'filter_currency_symbols' ) );

        // Set currency for PayPal recurring payments
        add_action( 'pms_paypal_subscr_new_payment_added', array( $this, 'set_currency_for_paypal_recurring_payments' ), 20, 3 );

        // Convert extra currency amount to base currency on payment completion
        add_action( 'pms_payment_update', array( $this, 'convert_multiple_currency_amount_to_base_currency' ), 30, 3 );

        // Set correct currency on Invoices
        add_filter( 'pms_invoice_custom_currency', array( $this, 'set_correct_currency_on_invoices' ), 20, 2 );

        // Add Multiple Currency data to Subscription Plan attributes for setting the correct currency on Tax Breakdown table
        add_filter( 'pms_get_subscription_plan_input_data_attrs', array( $this, 'add_subscription_plan_mc_data_attrs' ), 10, 3 );

        // Validate currency for PayPal payment gateway
        add_action( 'pms_register_form_validation', array( $this, 'validate_selected_currency_for_paypal' ) );
        add_action( 'pms_new_subscription_form_validation', array( $this, 'validate_selected_currency_for_paypal' ) );
        add_action( 'pms_upgrade_subscription_form_validation', array( $this, 'validate_selected_currency_for_paypal' ) );
        add_action( 'pms_change_subscription_form_validation', array( $this, 'validate_selected_currency_for_paypal' ) );
        add_action( 'pms_renew_subscription_form_validation', array( $this, 'validate_selected_currency_for_paypal' ) );
        add_action( 'pms_retry_payment_subscription_form_validation', array( $this, 'validate_selected_currency_for_paypal' ) );

    }

    /**
     * Set the currency data cookie
     *
     * @return void
     *
     */
    public function setup_currency_data_cookie(){

        if( is_admin() || wp_doing_cron() )
            return;

        if( isset( $_COOKIE['pms_mc_country_data'] ) )
            return;

        $country_data = pms_mc_get_current_user_country_data( apply_filters( 'pms_mc_user_ip_address', pms_get_user_ip_address() ) );

        pms_mc_set_country_cookie_data( $country_data );

    }

    /**
     * Front-end price output
     *
     * @param $output
     * @param $subscription_plan
     * @return mixed|string
     *
     */
    public function modify_price_output( $output, $subscription_plan ){

        if( $subscription_plan->price == 0 || !$this->should_change_currency( $subscription_plan->id ) )
            return $output;

        $pwyw_enabled = get_post_meta( $subscription_plan->id, 'pms_subscription_plan_pay_what_you_want', true );

        if ( $pwyw_enabled )
            return $output;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $currency          = $payment_settings['currency'];
        $price             = $subscription_plan->price;
        $mc_prices         = pms_mc_get_prices( $subscription_plan->id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $payment_settings['currency_switcher'] ) && isset( $_GET['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $currency = $selected_currency;
            $price = $mc_prices[$currency]['price'];
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $currency = $selected_currency;
            $price = pms_convert_currency( $subscription_plan->price, $payment_settings['currency'], $currency );
        }

        return '<span class="pms-divider"> - </span>' . pms_format_price( $price, $currency, array( 'before_price' => '<span class="pms-subscription-plan-price-value">', 'after_price' => '</span>', 'before_currency' => '<span class="pms-subscription-plan-currency">', 'after_currency' => '</span>' ) );

    }

    /**
     * Front-end sign-up fee output
     *
     * @param $output
     * @param $subscription_plan
     * @return mixed|string
     *
     */
    public function modify_signup_fee_output( $output, $subscription_plan ){

        if( $subscription_plan->sign_up_fee == 0 || !$this->should_change_currency( $subscription_plan->id ) )
            return $output;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $currency          = $payment_settings['currency'];
        $sign_up_fee       = $subscription_plan->sign_up_fee;
        $mc_prices         = pms_mc_get_prices( $subscription_plan->id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $payment_settings['currency_switcher'] ) && isset( $_GET['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );

        if( $conversion_type == 'manual' && ( empty( $selected_currency ) || empty( $mc_prices[ $selected_currency ] ) ) && !$auto_convert )
            return $output;

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ]['sign_up_fee'] ) ) {
            $currency = $selected_currency;
            $sign_up_fee = $mc_prices[ $selected_currency ]['sign_up_fee'];
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $currency = $selected_currency;
            $sign_up_fee = pms_convert_currency( $subscription_plan->sign_up_fee, $payment_settings['currency'], $currency );
        }
        elseif ( $conversion_type == 'manual' && empty( $mc_prices[ $selected_currency ]['sign_up_fee'] ) && !$auto_convert ) {
            return '';
        }

        return sprintf( __( ' and a %1$s sign-up fee', 'paid-member-subscriptions' ), pms_format_price( $sign_up_fee, $currency ) );

    }

    /**
     * Front-end Currency Switcher output on Register Form
     *
     * @param $data
     * @return mixed|string|void
     *
     */
    public function output_register_form_currency_switcher( $data ) {
        $active_design = pms_get_active_form_design();
        $payment_settings = get_option( 'pms_payments_settings', array() );


        if ( ( $active_design !== 'form-style-default' && !is_string( $data ) ) || !isset( $payment_settings['currency_switcher'] ) || $payment_settings['currency_switcher'] !== 'yes' )
            return $data;


        if ( $active_design === 'form-style-default' )
            $title = '<h3 class="pms-subscriptions-list-title">'. esc_html__( 'Select Your Subscription Plan', 'paid-member-subscriptions' ) .'</h3>';
        else $title = $data;


        $selected_currency = $payment_settings['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) ) {
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );
        }
        elseif ( isset( $payment_settings['user_location'] ) && $payment_settings['user_location'] == 'yes' ) {
            $country_data = pms_mc_get_country_cookie_data();

            if ( !empty( $country_data ) && !empty( $country_data['currency'] ) )
                $selected_currency = $country_data['currency'];
        }

        $title_box = '<div class="pms-title-container">';
            $title_box .= $title;

            $title_box .= '<div class="pms-currency-switcher-container">';

                $title_box .= '<label id="pms-currency-switcher-label" for="pms-currency-switcher">'. esc_html__( 'Currency', 'paid-member-subscriptions' ) .'</label>';

                $title_box .= '<select id="pms-currency-switcher" name="pms_mc_currency">';
                    $title_box .= '<option value="'. esc_attr( $payment_settings['currency'] ) .'" '. ( $payment_settings['currency'] == $selected_currency ? 'selected' : '' ) .' >'. esc_html( $payment_settings['currency'] ) .'</option>';

                    foreach ( $payment_settings['multiple-currencies'] as $currency ) {
                        $title_box .= '<option value="'. esc_attr( $currency ) .'" '. ( $currency == $selected_currency ? 'selected' : '' ) .'>'. esc_html( $currency ) .'</option>';
                    }

                $title_box .= '</select>';

            $title_box .= '</div>';

        $title_box .= '</div>';

        if ( $active_design === 'form-style-default' ) {
            echo $title_box; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
        }
        else return $title_box;
    }


    /**
     * Front-end Currency Switcher output on Change Subscription Form
     *
     * @param $location
     * @param $subscription_plan_upgrades
     * @param $subscription_plan_downgrades
     * @param $subscription_plan_others
     * @return void
     *
     */
    public function output_change_form_currency_switcher( $location, $subscription_plan_upgrades, $subscription_plan_downgrades, $subscription_plan_others ) {
        $payment_settings = get_option( 'pms_payments_settings', array() );
        $display_switcher = false;

        if( ( $location == 'upgrade_subscription' && !empty( $subscription_plan_upgrades ) ) ||
            ( $location == 'downgrade_subscription' && empty( $subscription_plan_upgrades ) && !empty( $subscription_plan_downgrades ) ) ||
            ( $location == 'change_subscription' && empty( $subscription_plan_upgrades ) && empty( $subscription_plan_downgrades ) && !empty( $subscription_plan_others ) ) ) {
            $display_switcher = true;
        }

        if ( !$display_switcher || !isset( $payment_settings['currency_switcher'] ) || $payment_settings['currency_switcher'] !== 'yes' )
            return;

        $selected_currency = $payment_settings['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) ) {
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );
        }
        elseif ( isset( $payment_settings['user_location'] ) && $payment_settings['user_location'] == 'yes' ) {
            $country_data = pms_mc_get_country_cookie_data();

            if ( !empty( $country_data ) && !empty( $country_data['currency'] ) )
                $selected_currency = $country_data['currency'];
        }

        $title_box = '<div class="pms-currency-switcher-container">';

            $title_box .= '<label id="pms-currency-switcher-label" for="pms-currency-switcher">'. esc_html__( 'Currency', 'paid-member-subscriptions' ) .'</label>';

            $title_box .= '<select id="pms-currency-switcher" name="pms_mc_currency">';
                $title_box .= '<option value="'. esc_attr( $payment_settings['currency'] ) .'" '. ( $payment_settings['currency'] == $selected_currency ? 'selected' : '' ) .' >'. esc_html( $payment_settings['currency'] ) .'</option>';

                foreach ( $payment_settings['multiple-currencies'] as $currency ) {
                    $title_box .= '<option value="'. esc_attr( $currency ) .'" '. ( $currency == $selected_currency ? 'selected' : '' ) .'>'. esc_html( $currency ) .'</option>';
                }

            $title_box .= '</select>';

        $title_box .= '</div>';

        echo $title_box; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
    }

    /**
     * Front-end Currency Switcher output on Renew Subscription and Retry Payment Forms
     *
     * @param $message
     * @param $subscription_plan
     * @param $member
     * @return mixed|string
     *
     */
    public function output_renew_retry_form_currency_switcher( $message, $subscription_plan, $member = null ) {
        $payment_settings = get_option( 'pms_payments_settings', array() );

        if ( !isset( $payment_settings['currency_switcher'] ) || $payment_settings['currency_switcher'] !== 'yes' )
            return $message;

        $selected_currency = $payment_settings['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) ) {
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );
        }
        elseif ( isset( $payment_settings['user_location'] ) && $payment_settings['user_location'] == 'yes' ) {
            $country_data = pms_mc_get_country_cookie_data();

            if ( !empty( $country_data ) && !empty( $country_data['currency'] ) )
                $selected_currency = $country_data['currency'];
        }

        $title_box = '<div class="pms-title-container">';
            $title_box .= $message;

            $title_box .= '<div class="pms-currency-switcher-container">';

                $title_box .= '<label id="pms-currency-switcher-label" for="pms-currency-switcher">'. esc_html__( 'Currency', 'paid-member-subscriptions' ) .'</label>';

                $title_box .= '<select id="pms-currency-switcher" name="pms_mc_currency">';
                    $title_box .= '<option value="'. esc_attr( $payment_settings['currency'] ) .'" '. ( $payment_settings['currency'] == $selected_currency ? 'selected' : '' ) .' >'. esc_html( $payment_settings['currency'] ) .'</option>';

                    foreach ( $payment_settings['multiple-currencies'] as $currency ) {
                        $title_box .= '<option value="'. esc_attr( $currency ) .'" '. ( $currency == $selected_currency ? 'selected' : '' ) .'>'. esc_html( $currency ) .'</option>';
                    }

                $title_box .= '</select>';

            $title_box .= '</div>';

        $title_box .= '</div>';

        return $title_box;
    }

    /**
     * Front-end Currency Switcher output on Profile Builder Form
     *
     * @param $title
     * @param $form_location
     * @return mixed|string
     *
     */
    public function output_pb_form_currency_switcher( $title, $form_location ) {

        if ( $form_location !== 'register' )
            return $title;

        $payment_settings = get_option( 'pms_payments_settings', array() );

        if ( !isset( $payment_settings['currency_switcher'] ) || $payment_settings['currency_switcher'] !== 'yes' )
            return $title;

        $selected_currency = $payment_settings['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) ) {
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );
        }
        elseif ( isset( $payment_settings['user_location'] ) && $payment_settings['user_location'] == 'yes' ) {
            $country_data = pms_mc_get_country_cookie_data();

            if ( !empty( $country_data ) && !empty( $country_data['currency'] ) )
                $selected_currency = $country_data['currency'];
        }

        $title_box = '<div class="pms-title-container">';
            $title_box .= $title;

            $title_box .= '<div class="pms-currency-switcher-container">';

                $title_box .= '<label id="pms-currency-switcher-label" for="pms-currency-switcher">'. esc_html__( 'Currency', 'paid-member-subscriptions' ) .'</label>';

                $title_box .= '<select id="pms-currency-switcher" name="pms_mc_currency">';
                    $title_box .= '<option value="'. esc_attr( $payment_settings['currency'] ) .'" '. ( $payment_settings['currency'] == $selected_currency ? 'selected' : '' ) .' >'. esc_html( $payment_settings['currency'] ) .'</option>';

                    foreach ( $payment_settings['multiple-currencies'] as $currency ) {
                        $title_box .= '<option value="'. esc_attr( $currency ) .'" '. ( $currency == $selected_currency ? 'selected' : '' ) .'>'. esc_html( $currency ) .'</option>';
                    }

                $title_box .= '</select>';

            $title_box .= '</div>';

        $title_box .= '</div>';

        return $title_box;
    }

    /**
     * Modify payment data before it's inserted into the database
     *
     * @param $payment_data
     * @param $checkout_data
     * @return mixed
     *
     */
    public function modify_checkout_payment_data( $payment_data, $checkout_data ){

        if( empty( $payment_data['amount'] ) || empty( $checkout_data['form_location'] ) || !$this->should_change_currency( $payment_data['subscription_plan_id'] ) )
            return $payment_data;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $payment_data['subscription_plan_id'] );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( function_exists( 'pms_in_pwyw_pricing_enabled' ) && pms_in_pwyw_pricing_enabled( $payment_data['subscription_plan_id'] ) ) {

            if( isset( $payment_data['payment_gateway'] ) && in_array( $payment_data['payment_gateway'], [ 'paypal_express', 'paypal_standard' ] ) && !$this->isPayPalRestrictedCurrency( $selected_currency ) )
                $payment_data['currency'] = $selected_currency;

            return $payment_data;
        }

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $price = $mc_prices[ $selected_currency ]['price'];

            if( !empty( $mc_prices[ $selected_currency ]['sign_up_fee'] ) && in_array( $checkout_data['form_location'], $this->get_sign_up_fee_form_locations() ) )
                $price = $price + $mc_prices[ $selected_currency ]['sign_up_fee'];

            // for PayPal gateways, some currencies are not accepted
            if( isset( $payment_data['payment_gateway'] ) && in_array( $payment_data['payment_gateway'], [ 'paypal_express', 'paypal_standard' ] ) && $this->isPayPalRestrictedCurrency( $selected_currency ) ){

                // update the payment_data amount if it can be converted to default currency
                if ( !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {

                    // convert it to base currency
                    $converted_amount = pms_convert_currency( $price, $selected_currency, pms_get_active_currency() );

                    $payment_data['amount'] = $converted_amount;

                    if( isset( $payment_data['sign_up_amount'] ) )
                        $payment_data['sign_up_amount'] = $converted_amount;

                }

            } else {
                $payment_data['amount'] = $price;
                $payment_data['currency'] = $selected_currency;

                if( isset( $payment_data['sign_up_amount'] ) )
                    $payment_data['sign_up_amount'] = $price;
            }
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {

            if( ! ( isset( $payment_data['payment_gateway'] ) && in_array( $payment_data['payment_gateway'], [ 'paypal_express', 'paypal_standard' ] ) && $this->isPayPalRestrictedCurrency( $selected_currency ) ) ) {
                $price = pms_convert_currency( $payment_data['amount'], $payment_settings['currency'], $selected_currency );
                $payment_data['amount'] = $price;
                $payment_data['currency'] = $selected_currency;

                if( isset( $payment_data['sign_up_amount'] ) )
                    $payment_data['sign_up_amount'] = $price;

            }

        }

        return $payment_data;

    }

    /**
     * Modify PSP recurring data
     *
     * @param $subscription_data
     * @param $checkout_data
     * @return array|mixed
     *
     */
    public function modify_checkout_subscription_data( $subscription_data = array(), $checkout_data = array() ){

        if( empty( $subscription_data ) || !$this->should_change_currency( $subscription_data['subscription_plan_id'] ) || empty( $subscription_data['subscription_plan_id'] ) )
            return $subscription_data;

        if ( function_exists( 'pms_in_pwyw_pricing_enabled' ) && pms_in_pwyw_pricing_enabled( $subscription_data['subscription_plan_id'] ) ) {
            return $subscription_data;
        }

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_data['subscription_plan_id'] );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );
        elseif ( isset( $_POST['subscription_id'] ) ) // set the correct currency when changing a subscription
            $selected_currency = pms_get_member_subscription_meta( sanitize_text_field( $_POST['subscription_id'] ), 'currency', true );

        if ( $conversion_type == 'manual' && !empty( $subscription_data['billing_amount'] ) && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $price = $mc_prices[ $selected_currency ]['price'];
            $subscription_data['billing_amount'] = $price;
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( $subscription_data['billing_amount'] ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $price = pms_convert_currency( $subscription_data['billing_amount'], $payment_settings['currency'], $selected_currency );
            $subscription_data['billing_amount'] = $price;
        }

        return $subscription_data;

    }

    /**
     * Save custom currency to payment
     *
     * @param $payment_gateway_data
     * @param $payments_settings
     * @return array|mixed
     *
     */
    public function save_custom_currency_to_payment( $payment_gateway_data, $payments_settings ){

        if( empty( $payment_gateway_data['payment_id'] ) )
            return $payment_gateway_data;

        $payment = pms_get_payment( absint( $payment_gateway_data['payment_id'] ) );

        if( !$this->should_change_currency( $payment->subscription_id ) )
            return $payment_gateway_data;

        $subscription_plan = pms_get_subscription_plan( $payment->subscription_id );
        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $payment->subscription_id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {

            if( isset( $payment_gateway_data['payment_gateway'] ) && in_array( $payment_gateway_data['payment_gateway'], [ 'paypal_express', 'paypal_standard' ] ) && $this->isPayPalRestrictedCurrency( $selected_currency ) ) {

                // set the default price if we cannot convert the custom price to default currency
                if ( !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {

                    // get the custom price for this currency
                    $initial_amount = ( function_exists( 'pms_in_pwyw_pricing_enabled' ) && pms_in_pwyw_pricing_enabled( $subscription_plan->id ) ) ? $payment->amount : $mc_prices[ $selected_currency ]['price'];

                    $target = isset( $_POST['pmstkn_original'] ) ? 'pmstkn_original' : 'pmstkn';
                    if( !empty( $mc_prices[ $selected_currency ]['sign_up_fee'] ) && in_array( PMS_Form_Handler::get_request_form_location( $target ), $this->get_sign_up_fee_form_locations() ) )
                        $initial_amount = $initial_amount + $mc_prices[ $selected_currency ]['sign_up_fee'];

                    // convert it to default currency
                    $converted_amount = pms_convert_currency( $initial_amount, $selected_currency, $payment_settings['currency'] );

                    if ( function_exists( 'pms_in_pwyw_pricing_enabled' ) && pms_in_pwyw_pricing_enabled( $payment_gateway_data['subscription_data']['subscription_plan_id'] ) ) {
                        $payment->update( array( 'amount' => $converted_amount ) );
                        $payment_gateway_data['amount'] = $converted_amount;

                        if ( isset( $payment_gateway_data['sign_up_amount'] ) )
                            $payment_gateway_data['sign_up_amount'] = $converted_amount;
                    }

                }
                else $converted_amount = $subscription_plan->price;

            }

            // Update subscription data billing amount to work with PayPal express recurring
            if( isset( $payment_gateway_data['subscription_data']['billing_amount'] ) && in_array( $payment_gateway_data['payment_gateway'], [ 'paypal_express' ] ) ){

                if( $this->isPayPalRestrictedCurrency( $selected_currency ) && isset( $converted_amount ) )
                    $payment_gateway_data['subscription_data']['billing_amount'] = $converted_amount;

            }

            if( isset( $initial_amount ) && isset( $converted_amount ) ) {

                // Add log entry
                $payment->log_data( 'pms_mc_currency_converted', [ 'from' => $initial_amount . ' ' . $selected_currency, 'to' => $converted_amount . ' ' . $payment_settings['currency'] ] );

            } else {

                // Save currency to payment and subscription
                $payment->update( array( 'currency' => $selected_currency ) );

                // Add log entry
                $payment->log_data( 'pms_mc_currency_switched', array( 'currency' => $selected_currency ) );

            }
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {

            if( isset( $payment_gateway_data['payment_gateway'] ) && in_array( $payment_gateway_data['payment_gateway'], [ 'paypal_express', 'paypal_standard' ] ) && $this->isPayPalRestrictedCurrency( $selected_currency ) ) {

                // convert back to base currency
                $initial_amount = $payment_gateway_data['subscription_data']['billing_amount'];
                $converted_amount = pms_convert_currency( $initial_amount, $selected_currency, $payment_settings['currency'] );

                if ( function_exists( 'pms_in_pwyw_pricing_enabled' ) && pms_in_pwyw_pricing_enabled( $payment_gateway_data['subscription_data']['subscription_plan_id'] ) ) {
                    $payment->update( array( 'amount' => $converted_amount ) );
                    $payment_gateway_data['amount'] = $converted_amount;

                    if ( isset( $payment_gateway_data['sign_up_amount'] ) )
                        $payment_gateway_data['sign_up_amount'] = $converted_amount;
                }
            }

            // Update subscription data billing amount to work with PayPal express recurring
            if( isset( $payment_gateway_data['subscription_data']['billing_amount'] ) && in_array( $payment_gateway_data['payment_gateway'], [ 'paypal_express' ] ) ) {

                if( $this->isPayPalRestrictedCurrency( $selected_currency ) && isset( $converted_amount ) )
                    $payment_gateway_data['subscription_data']['billing_amount'] = $converted_amount;

            }

            if( isset( $initial_amount ) && isset( $converted_amount ) ) {

                // Add log entry
                $payment->log_data( 'pms_mc_currency_converted', [ 'from' => $initial_amount . ' ' . $selected_currency, 'to' => $converted_amount . ' ' . $payment_settings['currency'] ] );

            }
            else {

                // Save currency to payment
                $payment->update( array( 'currency' => $selected_currency ) );

                // Add log entry
                $payment->log_data( 'pms_mc_currency_switched', array( 'currency' => $selected_currency ) );

            }

        }

        return $payment_gateway_data;

    }

    /**
     * Save currency on subscription when it's inserted
     *
     * @param $subscription
     * @param $checkout_data
     * @return void
     *
     */
    public function save_currency_to_subscription( $subscription, $checkout_data ){

        if( empty( $subscription->id ) || empty( $subscription->subscription_plan_id ) )
            return;

        if( !$this->should_change_currency( $subscription->subscription_plan_id ) )
            return;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription->subscription_plan_id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {

            if( !isset( $subscription->payment_gateway ) || !in_array( $subscription->payment_gateway, [ 'paypal_express', 'paypal_standard' ] ) || !$this->isPayPalRestrictedCurrency( $selected_currency ) )
                pms_update_member_subscription_meta( $subscription->id, 'currency', $selected_currency );

        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            pms_update_member_subscription_meta( $subscription->id, 'currency', $selected_currency );
        }


    }

    /**
     * Modify currency for the PWYW Subscription
     *
     * @param $currency
     * @param $subscription_plan_id
     * @return mixed|string
     *
     */
    public function modify_pwyw_subscription_currency( $currency, $subscription_plan_id ){

        if( !$this->should_change_currency( $subscription_plan_id ) )
            return $currency;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan_id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) ) {
            $currency = $selected_currency;
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $currency = $selected_currency;
        }

        return $currency;
    }

    /**
     * Update PWYW price based on MC Currency set price
     *
     * @param $amount
     * @param $subscription_plan_id
     * @return false|float|mixed
     *
     */
    public function modify_pwyw_subscription_price( $amount, $subscription_plan_id ){

        if( !$this->should_change_currency( $subscription_plan_id ) )
            return $amount;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan_id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $currency = $selected_currency;
            $amount = $mc_prices[$currency]['price'];
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) && !isset( $_POST['subscription_price_' . $subscription_plan_id] ) ) {
            $currency = $selected_currency;
            $price = $amount != 0 ? pms_convert_currency( $amount, $payment_settings['currency'], $currency ) : $amount;
            $amount = $price;
        }

        return $amount;
    }

    /**
     * Modify PWYW Subscription min_price for range validation
     *
     * @param $amount
     * @param $subscription_plan_id
     * @return false|float|mixed
     *
     */
    public function modify_pwyw_subscription_min_price( $amount, $subscription_plan_id ) {
        if( !$this->should_change_currency( $subscription_plan_id ) )
            return $amount;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan_id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['min_price'] ) ) {
            $amount = $mc_prices[ $selected_currency ]['min_price'];
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $min_price = $amount != 0 ? pms_convert_currency( $amount, $payment_settings['currency'], $selected_currency ) : $amount;
            $amount = $min_price;
        }

        return $amount;
    }

    /**
     * Modify PWYW Subscription max_price for range validation
     *
     * @param $amount
     * @param $subscription_plan_id
     * @return false|float|mixed
     *
     */
    public function modify_pwyw_subscription_max_price( $amount, $subscription_plan_id ) {
        if( !$this->should_change_currency( $subscription_plan_id ) )
            return $amount;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan_id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_GET['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['max_price'] ) ) {
            $amount = $mc_prices[ $selected_currency ]['max_price'];
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $max_price = $amount != 0 ? pms_convert_currency( $amount, $payment_settings['currency'], $selected_currency ) : $amount;
            $amount = $max_price;
        }

        return $amount;
    }

    /**
     * Modify currency in the Subscription Price merge tag
     *
     * @param $currency
     * @param $subscription_id
     * @return mixed
     *
     */
    public function modify_subscription_price_tag_currency( $currency, $subscription_id ){

        if( empty( $subscription_id ) )
            return $currency;

        $subscription_currency = pms_get_member_subscription_meta( $subscription_id, 'currency', true );

        if ( !empty( $subscription_currency ) )
            $currency = $subscription_currency;

        return $currency;

    }

    /**
     * Modify amount in the Subscription Price merge tag
     *  - taxes already included in the amount
     *
     * @param $amount_with_tax
     * @param $amount
     * @param $payment_id
     * @return mixed
     *
     */
    public function modify_subscription_price_tag_amount( $amount_with_tax, $amount, $payment_id ){

        // return initial amount (tax already included at this point)
        return $amount;

    }

    /**
     * Modify currency in the Payment History shortcode
     *
     * @param $currency
     * @param $payment_id
     * @return mixed|string
     *
     */
    public function modify_payment_history_amount_row_currency( $currency, $payment_id ){

        if(  empty( $payment_id ) )
            return $currency;

        $payment_currency = pms_mc_get_payment_currency( $payment_id );

        if( !empty( $payment_currency ) )
            $currency = $payment_currency;

        return $currency;

    }

    /**
     * Modify Subscription Plan amount in Stripe Intents checkout
     *
     * @param $amount
     * @param $subscription_plan
     * @return false|float|mixed
     *
     */
    public function modify_stripe_checkout_amount( $amount, $subscription_plan ){

        if( empty( $subscription_plan ) || empty( $subscription_plan->price ) || !$this->should_change_currency( $subscription_plan->id ) )
            return $amount;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan->id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];
        $target = isset( $_POST['pmstkn_original'] ) ? 'pmstkn_original' : 'pmstkn';

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $price = $mc_prices[ $selected_currency ]['price'];

            if( !empty( $mc_prices[ $selected_currency ]['sign_up_fee'] ) && in_array( PMS_Form_Handler::get_request_form_location( $target ), $this->get_sign_up_fee_form_locations() ) )
                $price = $price + $mc_prices[ $selected_currency ]['sign_up_fee'];

            $amount = $price;
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $price = pms_convert_currency( $amount, $payment_settings['currency'], $selected_currency );

            if( in_array( PMS_Form_Handler::get_request_form_location( $target ), $this->get_sign_up_fee_form_locations() ) )
                $price = $price + pms_convert_currency( $subscription_plan->sign_up_fee, $payment_settings['currency'], $selected_currency );

            $amount = $price;
        }

        return $amount;

    }

    /**
     * Disable Stripe sign-up fee calculation when prices are being changed as we add it ourselves
     *
     * @param $enabled
     * @param $subscription_plan
     * @return false|mixed
     *
     */
    public function disable_stripe_sign_up_fee( $enabled, $subscription_plan ){

        if( !$this->should_change_currency( $subscription_plan->id ) )
            return $enabled;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan->id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $enabled = false;
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $enabled = false;
        }

        return $enabled;

    }

    /**
     * Modify currency in Stripe Intents and Stripe Connect args
     *
     * @param $args
     * @param $subscription_plan
     * @return mixed
     *
     */
    public function modify_stripe_intent_args_currency( $args, $subscription_plan ){

        if( !$this->should_change_currency( $subscription_plan->id ) || empty( $subscription_plan ) )
            return $args;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan->id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $args['currency'] = $selected_currency;
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $args['currency'] = $selected_currency;
        }

        return $args;

    }

    /**
     * Modify currency in Stripe Intents args
     *
     * @param $args
     * @return mixed
     *
     */
    public function modify_stripe_intent_process_payment_args_currency( $args ){

        if( empty( $args['metadata'] ) || empty( $args['metadata']['subscription_id'] ) )
            return $args;

        $saved_currency = pms_get_member_subscription_meta( $args['metadata']['subscription_id'], 'currency', true );

        if( !empty( $saved_currency ) )
            $args['currency'] = $saved_currency;

        return $args;

    }

    /**
     * Modify currency in Stripe Connect -> update payment intent
     *
     * @param $currency
     * @param $subscription_plan
     * @param $payment_intent
     * @return mixed|string
     *
     */
    public function modify_stripe_connect_update_payment_intent_currency( $currency, $subscription_plan, $payment_intent ) {

        if( !$this->should_change_currency( $subscription_plan->id ) || empty( $subscription_plan ) )
            return $currency;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan->id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $currency = $selected_currency;
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $currency = $selected_currency;
        }

        return $currency;

    }

    /**
     * Modify currency in Next Payment Date account display
     *
     * @param $currency
     * @param $subscription
     * @return mixed
     *
     */
    public function modify_next_payment_date_currency( $currency, $subscription ){

        if( empty( $subscription->id ) )
            return $currency;

        $saved_currency = pms_get_member_subscription_meta( $subscription->id, 'currency', true );

        if( !empty( $saved_currency ) )
            return $saved_currency;

        return $currency;

    }

    /**
     * Modify currency in Member Subscription payment Data
     *
     * @param $payment_data
     * @param $subscription
     * @return mixed
     *
     */
    public function modify_psp_payments_currency( $payment_data, $subscription ){

        if( empty( $subscription->id ) )
            return $payment_data;

        $subscription_currency = pms_get_member_subscription_meta( $subscription->id, 'currency', true );

        if( !empty( $subscription_currency ) && $subscription_currency != $payment_data['currency'] )
            $payment_data['currency'] = $subscription_currency;

        return $payment_data;

    }

    /**
     * Save currency to payment meta for PSP added payments
     *
     * @param $subscription
     * @param $payment
     * @return void
     *
     */
    public function save_currency_to_payment_meta_for_psp_payments( $subscription, $payment ){

        if( empty( $subscription->id ) )
            return;

        $subscription_currency = pms_get_member_subscription_meta( $subscription->id, 'currency', true );

        if( !empty( $subscription_currency ) )
            pms_add_payment_meta( $payment->id, 'currency', $subscription_currency );

        return;

    }

    /**
     * Modify currency in Discount Codes message
     *
     * @param $currency
     * @param $subscription_plan
     * @return mixed|string
     *
     */
    public function modify_discounts_success_message_currency( $currency, $subscription_plan ){

        if( !$this->should_change_currency( $subscription_plan->id ) || empty( $subscription_plan->id ) )
            return $currency;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices($subscription_plan->id);
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $selected_currency ) && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $currency = $selected_currency;
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $currency = $selected_currency;
        }

        return $currency;

    }

    /**
     * Modify Subscription Plan price in Discount Code message
     *
     * @param $amount
     * @param $subscription_plan
     * @return false|float|mixed
     *
     */
    public function modify_discounts_success_message_plan_price( $amount, $subscription_plan ){

        if( !$this->should_change_currency( $subscription_plan->id ) || empty( $subscription_plan->id ) )
            return $amount;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = pms_mc_get_prices( $subscription_plan->id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $_POST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_POST['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $amount = $mc_prices[ $selected_currency ]['price'];
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $price = pms_convert_currency( $amount, $payment_settings['currency'], $selected_currency );
            $amount= $price;
        }

        return $amount;

    }

    /**
     * Validate discount code for the currency that is being used
     *
     * @param $error
     * @param $code
     * @param $subscription_plan_id
     * @param $discount_meta
     * @return mixed|string|null
     *
     */
    public function validate_discounts_currency_availability( $error, $code, $subscription_plan_id, $discount_meta ) {
        $country_data = pms_mc_get_country_cookie_data();
        $currency_availability = maybe_unserialize( isset( $discount_meta['pms_discount_currency_availability'][0] ) ? $discount_meta['pms_discount_currency_availability'][0] : '' );

        if ( isset( $_POST['pms_mc_currency'] ) ) {
            $currency = sanitize_text_field( $_POST['pms_mc_currency'] );
        }
        elseif ( $this->should_change_currency( $subscription_plan_id ) ) {
            $currency = $country_data['currency'];
        }
        else {
            $currency = pms_get_active_currency();
        }

        if ( !empty( $currency_availability ) && !in_array( $currency, $currency_availability ) )
            $error = esc_html__( 'This discount code is not available.', 'paid-member-subscriptions' );

        return $error;
    }

    /**
     * Modify Currency on PayPal Express Confirmation form
     *
     * @param $currency
     * @param $payment
     * @return mixed|string
     *
     */
    public function set_currency_on_paypal_express_confirmation_form( $currency, $payment ){

        if( empty( $payment->id ) )
            return $currency;

        $payment_currency = pms_mc_get_payment_currency( $payment->id );

        if( $payment_currency != $currency )
            $currency = $payment_currency;

        return $currency;

    }

    /**
     * Set currency for PayPal recurring payments
     *
     * @param $new_payment
     * @param $payment_data
     * @param $post_data
     * @return void
     *
     */
    public function set_currency_for_paypal_recurring_payments( $new_payment, $payment_data, $post_data ){

        if( empty( $new_payment->id ) || empty( $post_data['currency_code'] ) || empty( pms_get_active_currency_converter() ) || !pms_currency_rates_exist( $post_data['currency_code'] ) )
            return;

        $default_currency = pms_get_active_currency();

        if( $post_data['currency_code'] === $default_currency )
            return;

        pms_add_payment_meta( $new_payment->id, 'currency', $post_data['currency_code'] );

        // modify base currency amount because it cannot be calculated in the function below since that code runs before we set the correct currency for this payment
        $converted_amount = pms_convert_currency( $new_payment->amount, $post_data['currency_code'], $default_currency );
        
        pms_update_payment_meta( $new_payment->id, 'base_currency_amount', $converted_amount );

    }

    /**
     * Convert extra currency amount to base currency on payment completion
     *
     * @param $payment_id
     * @param $data
     * @param $old_data
     * @return void
     *
     */
    public function convert_multiple_currency_amount_to_base_currency( $payment_id, $data, $old_data ){
        
        if( empty( $data['status'] ) || $data['status'] != 'completed' || $data['status'] == $old_data['status'] )
            return;

        $subscription_id       = pms_get_payment_meta( $payment_id, 'subscription_id', true );
        $subscription_currency = pms_get_member_subscription_meta( $subscription_id, 'currency', true );
        $payment_currency      = pms_mc_get_payment_currency( $payment_id );
        $default_currency      = pms_get_active_currency();

        // save currency to subscription if empty
        if( empty( $subscription_currency ) && !empty( $payment_currency ) )
            pms_update_member_subscription_meta( $subscription_id, 'currency', $payment_currency );

        // This is for a PSP payment request
        if( empty( $payment_currency ) && wp_doing_cron() && !empty( $subscription_currency ) )
            $payment_currency = $subscription_currency;

        $payment = pms_get_payment( $payment_id );

        if( empty( $payment_currency ) || $payment_currency == $default_currency ){

            pms_update_payment_meta( $payment_id, 'base_currency_amount', $payment->amount );

        } elseif( !empty( $payment_currency ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $payment_currency ) ) {

            $converted_amount = pms_convert_currency( $payment->amount, $payment_currency, $default_currency );

            pms_update_payment_meta( $payment_id, 'base_currency_amount', $converted_amount );

        }
    }

    /**
     * Check if the Currency should be changed or not for a Subscription Plan
     *
     * @param $plan_id
     * @return bool
     *
     */
    public function should_change_currency( $plan_id = 0 ) {

        if( !empty( $plan_id ) ){
            $plan_excluded = get_post_meta( $plan_id, 'pms_plan_no_currency_switch', true );

            if( $plan_excluded == '1' )
                return false;

            $plan = pms_get_subscription_plan( $plan_id );

            if ( $plan->price == 0 && !$plan->has_sign_up_fee() )
                return false;
        }

        $country_data = pms_mc_get_country_cookie_data();

        if( empty( $country_data ) || empty( $country_data['currency'] ) )
            return false;

        $payment_settings = get_option( 'pms_payments_settings', array() );

        if ( isset( $payment_settings['currency_switcher'] ) && empty( $_REQUEST['pms_mc_currency'] ) && ( !isset( $payment_settings['user_location'] ) || $payment_settings['user_location'] != 'yes' ) )
            return false;

        $default_currency = pms_get_active_currency();
        $selected_currency = $country_data['currency'];

        if ( !empty( $_REQUEST['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_REQUEST['pms_mc_currency'] );

        if( $selected_currency == $default_currency || empty( $payment_settings['multiple-currencies'] ) || !in_array( $selected_currency, $payment_settings['multiple-currencies'] ) )
            return false;

        return true;
    }

    /**
     * Get the Form Locations where a SignUp Fee could be present
     *
     * @return string[]
     *
     */
    public function get_sign_up_fee_form_locations(){

        return array( 'register', 'new_subscription', 'retry_payment', 'register_email_confirmation', 'change_subscription' );

    }

    /**
     * Filter currency symbols so it shows the symbol for some currencies and the ISO slug for others
     *
     * @param $currency_symbols
     * @return mixed
     *
     */
    public function filter_currency_symbols( $currency_symbols ){
        $payment_settings = get_option( 'pms_payments_settings', array() );

        if( empty( $payment_settings['multiple-currencies-labels'] ) && !empty( $payment_settings['multiple-currencies'] ) ){

            foreach( $payment_settings['multiple-currencies'] as $currency ){
                if( isset( $currency_symbols[ $currency ] ) )
                    $currency_symbols[ $currency ] = $currency;
            }

        } elseif ( isset( $payment_settings['multiple-currencies-labels'] ) ) {

            foreach( $payment_settings['multiple-currencies-labels'] as $currency => $label ){
                if( isset( $currency_symbols[ $currency ] ) )
                    $currency_symbols[ $currency ] = !empty( $label ) ? $label : $currency;
            }

        }

        return $currency_symbols;
    }

    /**
     * Check if a specific currency is set as PayPal restricted currency
     *
     * @param $currency
     * @return bool
     *
     */
    private function isPayPalRestrictedCurrency( $currency ){
        $payment_settings = get_option( 'pms_payments_settings', array() );

        if( empty( $payment_settings ) || empty( $payment_settings['restricted-paypal-currencies'] ) )
            return false;

        if( in_array( $currency, $payment_settings['restricted-paypal-currencies'] ) )
            return true;

        return false;
    }

    /**
     * Set correct currency on Invoices
     *
     * @param $currency
     * @param $payment
     * @return mixed|string
     *
     */
    public function set_correct_currency_on_invoices( $currency, $payment ){

        if( empty( $payment ) || empty( $payment->id ) )
            return $currency;

        $payment_currency = pms_mc_get_payment_currency( $payment->id );

        if( !empty( $payment_currency ) )
            $currency = $payment_currency;

        return $currency;

    }

    /**
     * Add Multiple Currency data to Subscription Plan attributes for setting the correct currency on Tax Breakdown table
     *
     * @param $data_arr
     * @param $subscription_plan_id
     * @param $form_location
     * @return mixed
     *
     */
    public function add_subscription_plan_mc_data_attrs( $data_arr, $subscription_plan_id, $form_location ) {

        if( !is_array( $data_arr ) || $subscription_plan_id == 0 || !$this->should_change_currency( $subscription_plan_id ) )
            return $data_arr;

        $country_data      = pms_mc_get_country_cookie_data();
        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $currency          = $payment_settings['currency'];
        $price             = $data_arr['price'];
        $sign_up_fee       = isset( $data_arr['sign_up_fee'] ) ? $data_arr['sign_up_fee'] : 0;
        $mc_prices         = pms_mc_get_prices( $subscription_plan_id );
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];
        $selected_currency = $country_data['currency'];

        if ( isset( $payment_settings['currency_switcher'] ) && isset( $_GET['pms_mc_currency'] ) )
            $selected_currency = sanitize_text_field( $_GET['pms_mc_currency'] );

        if ( $conversion_type == 'manual' && !empty( $mc_prices[ $selected_currency ] ) && !empty( $mc_prices[ $selected_currency ]['price'] ) ) {
            $currency = $selected_currency;
            $price = $mc_prices[$currency]['price'];

            if ( !empty( $data_arr['sign_up_fee'] ) && !empty( $mc_prices[ $selected_currency ]['sign_up_fee'] )  )
                $sign_up_fee = (float) $mc_prices[ $selected_currency ]['sign_up_fee'];
        }
        elseif ( ( $conversion_type == 'automatic' || ( $conversion_type == 'manual' && $auto_convert ) ) && !empty( pms_get_active_currency_converter() ) && pms_currency_rates_exist( $selected_currency ) ) {
            $currency = $selected_currency;
            $price = pms_convert_currency( (float) $data_arr['price'], $payment_settings['currency'], $currency );

            if ( !empty( $data_arr['sign_up_fee'] ) )
                $sign_up_fee = pms_convert_currency( (float) $data_arr['sign_up_fee'], $payment_settings['currency'], $currency );
        }

        $data_arr['mc_price'] = $price;
        $data_arr['mc_currency'] = $this->filter_currency_symbols( $currency );
        $data_arr['mc_sign_up_fee'] = $sign_up_fee;
        $data_arr['mc_pwyw'] = function_exists( 'pms_in_pwyw_pricing_enabled' ) ? pms_in_pwyw_pricing_enabled( $subscription_plan_id ) : false;

        return $data_arr;
    }

    /**
     * Validate currency for PayPal payment gateway
     *
     * @return void
     */
    public function validate_selected_currency_for_paypal() {

        $payment_gateway  = isset( $_POST['pay_gate'] ) ? sanitize_text_field( $_POST['pay_gate'] ) : '';
        $payment_currency = isset( $_POST['pms_mc_currency'] ) ? sanitize_text_field( $_POST['pms_mc_currency'] ) : '';

        if ( empty( $payment_gateway ) || !in_array( $payment_gateway, [ 'paypal_express', 'paypal_standard' ] ) || empty( $payment_currency ) || !$this->isPayPalRestrictedCurrency( $payment_currency ) )
            return;

        $payment_settings  = get_option( 'pms_payments_settings', array() );
        $mc_prices         = isset( $_POST['subscription_plans'] ) ? pms_mc_get_prices( sanitize_text_field( $_POST['subscription_plans'] ) ) : array();
        $auto_convert      = isset( $payment_settings['auto_convert_prices'] ) && $payment_settings['auto_convert_prices'] == 'yes';
        $conversion_type   = $payment_settings['conversion_type'];

        if ( $conversion_type == 'manual' && !$auto_convert && ( empty( $payment_settings['convertor_api_provider'] ) || !pms_currency_rates_exist( $payment_currency ) ) && !empty( $mc_prices[ $payment_currency ] ) && !empty( $mc_prices[ $payment_currency ]['price'] ) ) {
            pms_errors()->add( 'form_general', __( 'The selected currency is not supported by PayPal, and conversion rates for this currency are unavailable. <br> Please reach out to the website administrator for further information.', 'paid-member-subscriptions' ) );
        }

    }
}

new PMS_Multiple_Currencies();