<?php

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

Class PMS_Alpha_Vantage_Currency_Converter extends PMS_Currency_Converter {

    /**
     * Set the Base URL for the conversion rates request
     *
     * @var string
     */
    private $base_url = 'https://www.alphavantage.co/query';

    /**
     * Set the required data
     *
     * @param $amount
     * @param $from
     * @param $to
     * @param $date
     */
    public function __construct( $amount, $from, $to, $date = '' ){

        parent::__construct( $amount, $from, $to, $date );

    }

    /**
     * Get the conversion rate between 2 currencies
     *
     * @return false|int
     */
    public function get_conversion_rate() {

        if ( $this->from != $this->to ) {
            $request = $this->make_request();

            if ( isset( $request->{"Realtime Currency Exchange Rate"} ) )
                $rate = $request->{"Realtime Currency Exchange Rate"}->{'5. Exchange Rate'};
            else $rate = null;

        }
        else $rate = 1;

        return $rate;

    }

    /**
     * Request the conversion rate between 2 currencies
     *
     * @return mixed|null
     */
    private function make_request() {

        $args = [
            'function'   => 'CURRENCY_EXCHANGE_RATE',
            'from_currency'   => $this->from,
            'to_currency'     => $this->to,
            'apikey'   => $this->get_api_key(),
        ];

        // LIMITED TO 25 REQUESTS PER DAY
        $url = add_query_arg( $args, $this->base_url );
        $response = file_get_contents( $url );

        return json_decode( $response );
    }

    /**
     * Get the Converter API Key
     *
     * @return mixed|string
     */
    private function get_api_key() {
        $payment_settings = get_option( 'pms_payments_settings', array() );
        $api_key = isset( $payment_settings['alpha_vantage_api_key'] ) ? $payment_settings['alpha_vantage_api_key'] : '';

        return $api_key;
    }
}