<?php

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

Class PMS_Exchange_Currency_Converter extends PMS_Currency_Converter {

    /**
     * Set the Base URL for the conversion rates request
     *
     * @var string
     */
    private $base_url = 'https://v6.exchangerate-api.com/v6';

    /**
     * Set the required data
     *
     * @param $amount
     * @param $from
     * @param $to
     * @param $date
     */
    public function __construct( $amount, $from, $to, $date = '' ){

        parent::__construct( $amount, $from, $to, $date );

    }

    /**
     * Get the conversion rate between 2 currencies
     *
     * @return false|int
     */
    public function get_conversion_rate() {

        if ( $this->from != $this->to ) {
            $request = $this->make_request();
            $rate = $request->conversion_rate;
        }
        else $rate = 1;

        return $rate;

    }

    /**
     * Request the conversion rate between 2 currencies
     *
     * @return mixed|null
     */
    private function make_request() {

        $args = [
            'api_key'   => $this->get_api_key(),
            'from'   => $this->from,
            'to'     => $this->to,
        ];

        $url = $this->base_url . '/' . $args['api_key'] . '/pair/' . $args['from'] . '/' . $args['to'];
        $response = file_get_contents( $url );

        return json_decode( $response );
    }

    /**
     * Get the Converter API Key
     *
     * @return mixed|string
     */
    private function get_api_key() {
        $payment_settings = get_option( 'pms_payments_settings', array() );
        $api_key = isset( $payment_settings['exchange_api_key'] ) ? $payment_settings['exchange_api_key'] : '';

        return $api_key;
    }
}