<?php

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

Class PMS_Frankfurter_Currency_Converter extends PMS_Currency_Converter {

    /**
     * Set the Base URL for the conversion rates request
     *
     * @var string
     */
    private $base_url = 'https://api.frankfurter.app/latest';

    /**
     * Set the required data
     *
     * @param $amount
     * @param $from
     * @param $to
     * @param $date
     */
    public function __construct( $amount, $from, $to, $date = '' ){

        parent::__construct( $amount, $from, $to, $date );

    }

    /**
     * Get the conversion rate between 2 currencies
     *
     * @return false|int
     */
    public function get_conversion_rate() {

        if ( $this->from != $this->to ) {
            $request = $this->make_request();

            if ( $request ) {
                $to = $this->to;
                $rate = $request->rates->$to;
            }
            else $rate = null;
        }
        else $rate = 1;

        return $rate;

    }

    /**
     * Request the conversion rate between 2 currencies
     *
     * @return mixed|null
     */
    private function make_request() {

        // get an array with the supported currencies
        $currency_list = get_object_vars( json_decode( file_get_contents( 'https://api.frankfurter.app/currencies' ) ) );

        if ( !isset( $currency_list[ $this->from ] ) || !isset( $currency_list[ $this->to ] ) )
            return false;

        $args = [
            'base' => $this->from,
            'symbols' => $this->to,
        ];

        $url = add_query_arg( $args, $this->base_url );

        $response = file_get_contents( $url );

        return json_decode( $response );
    }

}