<?php

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

// Return if PMS is not active
if( ! defined( 'PMS_VERSION' ) ) return;

/**
 * Get the manually set extra currency prices for a specific Subscription Plan
 *
 * @param $subscription_plan_id
 * @return array|mixed
 */
function pms_mc_get_prices( $subscription_plan_id ){
    
    if( empty( $subscription_plan_id ) )
        return array();

    $prices = get_post_meta( $subscription_plan_id, 'pms_mc_prices', true );

    if( empty( $prices ) )
        return array();

    return $prices;

}

/**
 * Get current user Country data
 *
 * @param $ip_address
 * @return array|false
 * @throws \MaxMind\Db\Reader\InvalidDatabaseException
 * @throws \PMS_MC_MaxMind\Db\Reader\InvalidDatabaseException
 */
function pms_mc_get_current_user_country_data( $ip_address ){

    if( empty( $ip_address ) )
        return false;

    include_once( __DIR__ . '/../assets/lib/autoload.php' );
    include_once( __DIR__ . '/../assets/lib/PMS_MC_MaxMind/Db/Reader.php' );
    include_once( __DIR__ . '/../assets/lib/PMS_MC_MaxMind/Db/Reader/Decoder.php' );
    include_once( __DIR__ . '/../assets/lib/PMS_MC_MaxMind/Db/Reader/InvalidDatabaseException.php' );
    include_once( __DIR__ . '/../assets/lib/PMS_MC_MaxMind/Db/Reader/Metadata.php' );
    include_once( __DIR__ . '/../assets/lib/PMS_MC_MaxMind/Db/Reader/Util.php' );

    $db_path = __DIR__ . '/../assets/lib/GeoLite2-Country/GeoLite2-Country.mmdb';
    $reader  = new PMS_MC_MaxMind\Db\Reader( $db_path );
    $data    = false;

    $record  = $reader->get( $ip_address );

    if( !empty( $record ) && isset( $record['country'] ) && isset( $record['country']['iso_code'] ) ){

        $data = [
            'ip_address'    => $ip_address,
            'country'       => $record['country']['iso_code'],
            'country_label' => isset( $record['country']['names']['en'] ) ? $record['country']['names']['en'] : '',
            'currency'      => pms_mc_get_country_currency( $record['country']['iso_code'] ),
        ];

    }

    return $data;

}

/**
 * Get country specific currency codes
 *
 * @param $country_code
 * @return false|mixed
 */
function pms_mc_get_country_currency( $country_code ) {
    
    $data = array(
        'AF' => 'AFN',
        'AL' => 'ALL',
        'DZ' => 'DZD',
        'AS' => 'USD',
        'AD' => 'EUR',
        'AO' => 'AOA',
        'AI' => 'XCD',
        'AQ' => 'XCD',
        'AG' => 'XCD',
        'AR' => 'ARS',
        'AM' => 'AMD',
        'AW' => 'AWG',
        'AU' => 'AUD',
        'AT' => 'EUR',
        'AZ' => 'AZN',
        'BS' => 'BSD',
        'BH' => 'BHD',
        'BD' => 'BDT',
        'BB' => 'BBD',
        'BY' => 'BYR',
        'BE' => 'EUR',
        'BZ' => 'BZD',
        'BJ' => 'XOF',
        'BM' => 'BMD',
        'BT' => 'BTN',
        'BO' => 'BOB',
        'BA' => 'BAM',
        'BW' => 'BWP',
        'BV' => 'NOK',
        'BR' => 'BRL',
        'IO' => 'USD',
        'BN' => 'BND',
        'BG' => 'BGN',
        'BF' => 'XOF',
        'BI' => 'BIF',
        'KH' => 'KHR',
        'CM' => 'XAF',
        'CA' => 'CAD',
        'CV' => 'CVE',
        'KY' => 'KYD',
        'CF' => 'XAF',
        'TD' => 'XAF',
        'CL' => 'CLP',
        'CN' => 'CNY',
        'HK' => 'HKD',
        'CX' => 'AUD',
        'CC' => 'AUD',
        'CO' => 'COP',
        'KM' => 'KMF',
        'CG' => 'XAF',
        'CD' => 'CDF',
        'CK' => 'NZD',
        'CR' => 'CRC',
        'HR' => 'HRK',
        'CU' => 'CUP',
        'CY' => 'EUR',
        'CZ' => 'CZK',
        'DK' => 'DKK',
        'DJ' => 'DJF',
        'DM' => 'XCD',
        'DO' => 'DOP',
        'EC' => 'ECS',
        'EG' => 'EGP',
        'SV' => 'SVC',
        'GQ' => 'XAF',
        'ER' => 'ERN',
        'EE' => 'EUR',
        'ET' => 'ETB',
        'FK' => 'FKP',
        'FO' => 'DKK',
        'FJ' => 'FJD',
        'FI' => 'EUR',
        'FR' => 'EUR',
        'GF' => 'EUR',
        'TF' => 'EUR',
        'GA' => 'XAF',
        'GM' => 'GMD',
        'GE' => 'GEL',
        'DE' => 'EUR',
        'GH' => 'GHS',
        'GI' => 'GIP',
        'GR' => 'EUR',
        'GL' => 'DKK',
        'GD' => 'XCD',
        'GP' => 'EUR',
        'GU' => 'USD',
        'GT' => 'QTQ',
        'GG' => 'GGP',
        'GN' => 'GNF',
        'GW' => 'GWP',
        'GY' => 'GYD',
        'HT' => 'HTG',
        'HM' => 'AUD',
        'HN' => 'HNL',
        'HU' => 'HUF',
        'IS' => 'ISK',
        'IN' => 'INR',
        'ID' => 'IDR',
        'IR' => 'IRR',
        'IQ' => 'IQD',
        'IE' => 'EUR',
        'IM' => 'GBP',
        'IL' => 'ILS',
        'IT' => 'EUR',
        'JM' => 'JMD',
        'JP' => 'JPY',
        'JE' => 'GBP',
        'JO' => 'JOD',
        'KZ' => 'KZT',
        'KE' => 'KES',
        'KI' => 'AUD',
        'KP' => 'KPW',
        'KR' => 'KRW',
        'KW' => 'KWD',
        'KG' => 'KGS',
        'LA' => 'LAK',
        'LV' => 'EUR',
        'LB' => 'LBP',
        'LS' => 'LSL',
        'LR' => 'LRD',
        'LY' => 'LYD',
        'LI' => 'CHF',
        'LT' => 'EUR',
        'LU' => 'EUR',
        'MK' => 'MKD',
        'MG' => 'MGF',
        'MW' => 'MWK',
        'MY' => 'MYR',
        'MV' => 'MVR',
        'ML' => 'XOF',
        'MT' => 'EUR',
        'MH' => 'USD',
        'MQ' => 'EUR',
        'MR' => 'MRO',
        'MU' => 'MUR',
        'YT' => 'EUR',
        'MX' => 'MXN',
        'FM' => 'USD',
        'MD' => 'MDL',
        'MC' => 'EUR',
        'MN' => 'MNT',
        'ME' => 'EUR',
        'MS' => 'XCD',
        'MA' => 'MAD',
        'MZ' => 'MZN',
        'MM' => 'MMK',
        'NA' => 'NAD',
        'NR' => 'AUD',
        'NP' => 'NPR',
        'NL' => 'EUR',
        'AN' => 'ANG',
        'NC' => 'XPF',
        'NZ' => 'NZD',
        'NI' => 'NIO',
        'NE' => 'XOF',
        'NG' => 'NGN',
        'NU' => 'NZD',
        'NF' => 'AUD',
        'MP' => 'USD',
        'NO' => 'NOK',
        'OM' => 'OMR',
        'PK' => 'PKR',
        'PW' => 'USD',
        'PA' => 'PAB',
        'PG' => 'PGK',
        'PY' => 'PYG',
        'PE' => 'PEN',
        'PH' => 'PHP',
        'PN' => 'NZD',
        'PL' => 'PLN',
        'PT' => 'EUR',
        'PR' => 'USD',
        'QA' => 'QAR',
        'RE' => 'EUR',
        'RO' => 'RON',
        'RU' => 'RUB',
        'RW' => 'RWF',
        'SH' => 'SHP',
        'KN' => 'XCD',
        'LC' => 'XCD',
        'PM' => 'EUR',
        'VC' => 'XCD',
        'WS' => 'WST',
        'SM' => 'EUR',
        'ST' => 'STD',
        'SA' => 'SAR',
        'SN' => 'XOF',
        'RS' => 'RSD',
        'SC' => 'SCR',
        'SL' => 'SLL',
        'SG' => 'SGD',
        'SK' => 'EUR',
        'SI' => 'EUR',
        'SB' => 'SBD',
        'SO' => 'SOS',
        'ZA' => 'ZAR',
        'GS' => 'GBP',
        'SS' => 'SSP',
        'ES' => 'EUR',
        'LK' => 'LKR',
        'SD' => 'SDG',
        'SR' => 'SRD',
        'SJ' => 'NOK',
        'SZ' => 'SZL',
        'SE' => 'SEK',
        'CH' => 'CHF',
        'SY' => 'SYP',
        'TW' => 'TWD',
        'TJ' => 'TJS',
        'TZ' => 'TZS',
        'TH' => 'THB',
        'TG' => 'XOF',
        'TK' => 'NZD',
        'TO' => 'TOP',
        'TT' => 'TTD',
        'TN' => 'TND',
        'TR' => 'TRY',
        'TM' => 'TMT',
        'TC' => 'USD',
        'TV' => 'AUD',
        'UG' => 'UGX',
        'UA' => 'UAH',
        'AE' => 'AED',
        'GB' => 'GBP',
        'US' => 'USD',
        'UM' => 'USD',
        'UY' => 'UYU',
        'UZ' => 'UZS',
        'VU' => 'VUV',
        'VE' => 'VEF',
        'VN' => 'VND',
        'VI' => 'USD',
        'WF' => 'XPF',
        'EH' => 'MAD',
        'YE' => 'YER',
        'ZM' => 'ZMW',
        'ZW' => 'ZWD',
    );

    if( isset( $data[$country_code] ) )
        return $data[$country_code];

    return false;

}

/**
 * Get Payment currency
 *
 * @param $payment_id
 * @return mixed|string
 */
function pms_mc_get_payment_currency( $payment_id ) {
    $payment = pms_get_payment( absint( $payment_id ) );
    $payment_data = $payment->get_data( $payment->id );

    $currency = !empty( $payment_data['currency'] ) ? $payment_data['currency'] : pms_get_payment_meta( $payment->id, 'currency', true );

    if ( empty( $currency ) ) {
        $currency = pms_get_active_currency();
    }

    return $currency;
}

/**
 * Set Country data Cookie
 *
 * @param $country_data
 * @return void
 */
function pms_mc_set_country_cookie_data( $country_data ){

    if( !empty( $country_data ) )
        setcookie( 'pms_mc_country_data', base64_encode( json_encode( $country_data ) ), time() + 43200, '/' );

}

/**
 * Get Country data Cookie
 *
 * @return array|false|mixed|null
 * @throws \MaxMind\Db\Reader\InvalidDatabaseException
 * @throws \PMS_MC_MaxMind\Db\Reader\InvalidDatabaseException
 */
function pms_mc_get_country_cookie_data(){

    if( isset( $_COOKIE['pms_mc_country_data'] ) )
        return json_decode( base64_decode( sanitize_text_field( $_COOKIE['pms_mc_country_data'] ) ), true );

    $country_data = pms_mc_get_current_user_country_data( apply_filters( 'pms_mc_user_ip_address', pms_get_user_ip_address() ) );

    if( !empty( $country_data ) )
        return $country_data;

    return false;

}

/**
 * Shortcode for displaying the price for a specific Subscription Plan based on User Location
 *
 * @param $atts
 * @return string|void
 * @throws \MaxMind\Db\Reader\InvalidDatabaseException
 * @throws \PMS_MC_MaxMind\Db\Reader\InvalidDatabaseException
 */
function pms_mc_get_plan_price( $atts ) {

    $atts = shortcode_atts( array(
        'plan_id' => '',
    ), $atts, 'pms-mc-get-plan-price' );

    if( empty( $atts['plan_id'] ) )
        return '';

    $subscription_plan = pms_get_subscription_plan( absint( $atts['plan_id'] ) );

    if( empty( $subscription_plan->id ) )
        return '';

    $default_currency = pms_get_active_currency();
    $country_data     = pms_mc_get_country_cookie_data();

    if( empty( $country_data ) || $country_data['currency'] == $default_currency )
        return '';

    // check if plan has setup different pricing for the current detected currency
    $mc_prices = pms_mc_get_prices( $subscription_plan->id );

    if( !empty( $country_data['currency'] ) && !empty( $mc_prices[ $country_data['currency'] ] ) && !empty( $mc_prices[ $country_data['currency'] ]['price'] ) )
        return pms_format_price( $mc_prices[ $country_data['currency'] ]['price'], $country_data['currency'] );

}
add_shortcode( 'pms-mc-get-plan-price', 'pms_mc_get_plan_price' );

/**
 * Enqueue MC Scripts and Styles
 *
 * @return void
 */
function pms_mc_enqueue_front_end_scripts() {
    wp_enqueue_style( 'pms_mc_front_end_style', PMS_IN_MC_PLUGIN_DIR_URL . 'assets/css/front-end.css', array(),PMS_VERSION );
    wp_enqueue_script( 'pms_mc_front_end_script', PMS_IN_MC_PLUGIN_DIR_URL . 'assets/js/front-end.js', array( 'jquery' ),PMS_VERSION );
}
add_action('wp_enqueue_scripts' , 'pms_mc_enqueue_front_end_scripts');