<?php

/**
 * Check if Multiple Currencies Add-On is enabled before activating Pro Rate
 * - Multiple Currencies are not compatible with Pro Rate
 *
 * @param $slug
 * @return void
 */
function pms_pro_rate_activation_check( $slug ) {

    if ( $slug != 'pms-add-on-pro-rate/index.php' )
        return;

    $pms_add_ons_settings = get_option( 'pms_add_ons_settings', array() );
    $multiple_currencies_add_on_active = !empty( $pms_add_ons_settings['pms-add-on-multiple-currencies/index.php'] ) ? $pms_add_ons_settings['pms-add-on-multiple-currencies/index.php'] : false;

    if( $multiple_currencies_add_on_active ){

        $activation_failed_data['failed_activation'] = true;
        $activation_failed_data['message'] = esc_html__( 'The Pro-Rate Add-On is not compatible with the Multiple Currencies Add-On. To use Pro-Rate, please deactivate the Multiple Currencies Add-On.', 'paid-member-subscriptions' );
        update_option( 'pms_pro_rate_activation_failed', $activation_failed_data );

        $pms_add_ons_settings['pms-add-on-pro-rate/index.php'] = false;
        update_option( 'pms_add_ons_settings', $pms_add_ons_settings );

    }
    else delete_option( 'pms_pro_rate_activation_failed' );

}
add_action( 'pms_add_ons_activate', 'pms_pro_rate_activation_check' );


/**
 * Handle Pro Rate Add-On incompatibility admin notice
 *
 * @return void
 */
function pms_pro_rate_compatibility_notice() {

    if ( defined('DOING_AJAX') && DOING_AJAX )
        return;

    $failed_activation = get_option( 'pms_pro_rate_activation_failed', false );

    if ( !$failed_activation )
        return;

    $pms_add_ons_settings = get_option( 'pms_add_ons_settings', array() );
    $pro_rate_add_on_active = !empty( $pms_add_ons_settings['pms-add-on-pro-rate/index.php'] ) ? $pms_add_ons_settings['pms-add-on-pro-rate/index.php'] : false;
    $multiple_currencies_add_on_active = !empty( $pms_add_ons_settings['pms-add-on-multiple-currencies/index.php'] ) ? $pms_add_ons_settings['pms-add-on-multiple-currencies/index.php'] : false;

    if ( !$pro_rate_add_on_active && !$multiple_currencies_add_on_active ) {
        delete_option( 'pms_pro_rate_activation_failed' );
        return;
    }

    if ( !empty( $failed_activation['message'] ) ) {
        $pms_notifications_instance = PMS_Plugin_Notifications::get_instance();
        $pms_notifications_instance->add_notification( 'pms_pro_rate_incompatibility', $failed_activation['message'], 'pms-notice error notice is-dismissible', false );
    }

}
add_action( 'admin_init', 'pms_pro_rate_compatibility_notice' );